﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Threading;
using System.Xml.Linq;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;

using FieldApp.Extensibility;
using QueryAddin;
using EditAddin;

namespace ElectricAddin
{

	public class TraceTool : IMapTool
	{

		private const string _sIconPath = "ElectricAddin.Images.TraceOut.png";
		private const string _sCursorPath = "ElectricAddin.Images.Flag.cur";
		private bool _bInitialized = false;
		private bool _bEnabled = false;
		private Draw _Draw;

		#region IMapTool implementation
		public event EventHandler<EventArgs> IsEnabledChanged;

		public object ButtonContent
		{
			get { return AddinHelper.GetImage(Assembly.GetExecutingAssembly(), _sIconPath); }
		}

		public System.Windows.Input.Cursor MouseCursor
		{
			get { return AddinHelper.GetCursor(Assembly.GetExecutingAssembly(), _sCursorPath); }
		}

		public string ToolTip
		{
			get { return "Protective Device Trace"; }
		}

		public bool IsEnabled
		{
			get
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					TraceBehavior tb = TraceBehavior.Current;
					if (tb.IsInitialized)
						_bEnabled = true;
					else
						tb.Initialized += tb_Initialized;
				}
				return _bEnabled;
			}
		}

		public void Activate()
		{
			if (_Draw == null)
			{
				Map MyMap = MapApplication.Current.Map;
				_Draw = new Draw(MyMap);
				_Draw.DrawMode = DrawMode.Point;
				_Draw.IsEnabled = false;
				_Draw.DrawComplete += _Draw_DrawComplete;
			}
			_Draw.IsEnabled = true;
		}

		public void Deactivate()
		{
			if (_Draw != null)
				_Draw.IsEnabled = false;
		}
		#endregion

		void tb_Initialized(object sender, EventArgs e)
		{
			_bEnabled = true;
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

		void _Draw_DrawComplete(object sender, DrawEventArgs e)
		{
			MapPoint pt = (MapPoint)e.Geometry;
			TraceBehavior tb = TraceBehavior.Current;
			tb.DoTrace(pt);
		}

	}

	public class ClearTraceButton : IMapButton
	{

		private const string _sIconPath = "ElectricAddin.Images.Clear.png";

		public object ButtonContent
		{
			get { return AddinHelper.GetImage(Assembly.GetExecutingAssembly(), _sIconPath); }
		}

		public bool IsEnabled
		{
			get { return true; }
		}

		public string ToolTip
		{
			get { return "Clear Trace Results"; }
		}

		public void OnClick()
		{
			if (TraceBehavior.ResultsLayer == null)
				return;
			TraceBehavior.ResultsLayer.Graphics.Clear();
		}

		public event EventHandler<EventArgs> IsEnabledChanged { add { } remove { } } // Not used  

	}

	public class MarkupTool : EditSplitTool
	{

		private enum MarkupType
		{
			Point,
			Line,
			Poly
		}

		private const string sPointIcon = "ElectricAddin.Images.RedPoint.png";
		private const string sLineIcon = "ElectricAddin.Images.RedLine.png";
		private const string sPolyIcon = "ElectricAddin.Images.RedPoly.png";
		private const string sWindowIcon = "ElectricAddin.Images.Edit.ico";

		private MarkupType _MarkupType;
		private Draw _Draw;
		private clsFieldHelper _fh;

		public MarkupTool()
		{
			_MyAssembly = Assembly.GetExecutingAssembly();
			_sButtonContent = null;
			_sIconPath = sPointIcon;
			_sCursorPath = null;
			_sDropIconPath = "ElectricAddin.Images.Down.png";
			_sToolTip = "Point Markup";
			_MarkupType = MarkupType.Point;
		}

		protected override void EdInitialized()
		{
			Map MyMap = MapApplication.Current.Map;
			_Draw = new Draw(MyMap);
			_Draw.DrawMode = DrawMode.Point;
			_Draw.IsEnabled = false;
			_Draw.DrawComplete += _Draw_DrawComplete;
			_fh = EditBehavior.FieldHelper;
		}

		protected override void EdActivateTool()
		{
			_Draw.IsEnabled = true;
		}

		protected override void EdDeactivateTool()
		{
			_Draw.IsEnabled = false;
		}

		protected override void EdOnItemSelected(IMapMenuItem item)
		{
			_Draw.IsEnabled = false;
			if (item is RedPointMenuItem)
			{
				_MarkupType = MarkupType.Point;
				_Draw.DrawMode = DrawMode.Point;
				_sIconPath = sPointIcon;
				_sToolTip = "Point Markup";
			}
			else if (item is RedLineMenuItem)
			{
				_MarkupType = MarkupType.Line;
				_Draw.DrawMode = DrawMode.Polyline;
				_sIconPath = sLineIcon;
				_sToolTip = "Line Markup";
			}
			else
			{
				_MarkupType = MarkupType.Poly;
				_Draw.DrawMode = DrawMode.Polygon;
				_sIconPath = sPolyIcon;
				_sToolTip = "Polygon Markup";
			}
			_Draw.IsEnabled = true;
		}

		void _Draw_DrawComplete(object sender, DrawEventArgs e)
		{

			// Launch markup dialog

			string sLayerName;
			switch (_MarkupType)
			{
				case MarkupType.Point:
					sLayerName = "RedPoint";
					break;
				case MarkupType.Line:
					sLayerName = "RedLine";
					break;
				default:
					sLayerName = "RedPoly";
					break;
			}
			ucAddElectricMarkup uc = new ucAddElectricMarkup();
			ParentWindow pw = new ParentWindow();
			pw.SetTitle("Add Markup: " + sLayerName);
			pw.SetIcon(AddinHelper.GetImageSource(_MyAssembly, sWindowIcon));
			pw.SetContent(uc, false);
			pw.SizeToContent = System.Windows.SizeToContent.WidthAndHeight;
			uc.Initialize(pw);
			if (!pw.OpenDialog())
				return;
			Dictionary<string, object> Values = uc.Values;

			// Create markup

			XElement xFeat = clsFeatureHelper.CreateXElement(e.Geometry, Values);
			_fh.ProcessLocalFeature(sLayerName, xFeat, true);
			EditBehavior.LocalLayer.Refresh();

		}



	}

	


}
