﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using System.Windows.Controls;
using System.Windows.Input;
//using System.Windows.Media;
using System.Windows.Media.Imaging;

using FieldApp.Extensibility;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Local;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Symbols;
using ESRI.ArcGIS.Client.Tasks;

namespace ElectricAddin
{

	public class TraceBehavior : IBehavior
	{

		public static TraceBehavior Current { get; private set; }
		public static GraphicsLayer ResultsLayer { get; private set; }
		public bool IsInitialized { get; private set; }

		private const string _sTracePackage = @"C:\apps\FieldEdit\packages\ProtectiveTrace.gpk";
		private const string _sDynamicMapLayerName = "AZ Electric";
		private const string _sResultsLayerName = "TraceResultsLayer";
		private const string _sOHEdgeLayerName = "OH Primary Conductor";
		private const string _sUGEdgeLayerName = "UG Primary Conductor";
		private const string _sWorkspace = @"C:\apps\FieldEdit\Data\AZElectric.gdb";

		private Map _Map;
		private ArcGISDynamicMapServiceLayer _MapLayer;
		private List<int> _EdgeLayerIDs;
		private LocalGeoprocessingService _localTraceService;
		private MapPoint _ptIdentify;

		public event EventHandler<EventArgs> Initialized;

		#region Initialization
		public void OnAttached()
		{
			Current = this;
			_Map = MapApplication.Current.Map;
			LayerCollection MyLayers = _Map.Layers;
			MyLayers.LayersInitialized += MyLayers_LayersInitialized;
		}

		void MyLayers_LayersInitialized(object sender, EventArgs args)
		{

			// Get layers and edge IDs

			_MapLayer = null;
			ResultsLayer = null;
			_EdgeLayerIDs = new List<int>();
			LayerCollection MyLayers = _Map.Layers;
			foreach (Layer l in MyLayers)
			{
				if (l is GraphicsLayer)
				{
					if (l.ID != _sResultsLayerName)
						continue;
					ResultsLayer = (GraphicsLayer)l;
					continue;
				}
				if (!(l is ArcGISDynamicMapServiceLayer))
					continue;
				if (l.ID != _sDynamicMapLayerName)
					continue;
				_MapLayer = (ArcGISDynamicMapServiceLayer)l;
				foreach (LayerInfo li in _MapLayer.Layers)
				{
					if ((li.Name != _sOHEdgeLayerName) && (li.Name != _sUGEdgeLayerName))
						continue;
					_EdgeLayerIDs.Add(li.ID);
				}
			}
			if (ResultsLayer == null)
				return;
			if (_MapLayer == null)
				return;
			if (_EdgeLayerIDs.Count == 0)
				return;

			// Fire up geoprocessing service

			_localTraceService = new LocalGeoprocessingService(_sTracePackage, GPServiceType.Execute);
			_localTraceService.StartCompleted += new EventHandler<System.ComponentModel.AsyncCompletedEventArgs>(_localTraceService_StartCompleted);
			_localTraceService.StartAsync();

		}

		void _localTraceService_StartCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
		{
			IsInitialized = true;
			this.Initialized(this, new EventArgs());
		}
		#endregion

		#region 

		public void DoTrace(MapPoint pt)
		{

			// Build geometry for identify task

			MapApplication.Current.Status("Finding network feature...", true);
			_ptIdentify = pt;
			double d = _Map.Resolution * 5; // Pixel tolerance
			Envelope env = pt.Extent;
			if (env.Width == 0)
			{
				env.XMin -= d;
				env.XMax += d;
			}
			if (env.Height == 0)
			{
				env.YMin -= d;
				env.YMax += d;
			}

			// Use identify task to get network feature geometry

			IdentifyParameters identifyParameters = new IdentifyParameters();
			identifyParameters.LayerOption = LayerOption.all;
			List<int> LayerIDs = identifyParameters.LayerIds;
			foreach (int id in _EdgeLayerIDs)
				LayerIDs.Add(id);
			identifyParameters.MapExtent = _Map.Extent;
			identifyParameters.Width = (int)_Map.ActualWidth;
			identifyParameters.Height = (int)_Map.ActualHeight;
			identifyParameters.Geometry = env;
			IdentifyTask it = new IdentifyTask();
			it.Url = _MapLayer.Url;
			it.ExecuteCompleted += new EventHandler<IdentifyEventArgs>(it_ExecuteCompleted);
			it.Failed += new EventHandler<TaskFailedEventArgs>(it_Failed);
			it.ExecuteAsync(identifyParameters);
			
		}

		private void it_ExecuteCompleted(object sender, IdentifyEventArgs e)
		{

			// Check for returned feature

			bool bFound = true;
			if (e == null)
				bFound = false;
			else if (e.IdentifyResults == null)
				bFound = false;
			else if (e.IdentifyResults.Count == 0)
				bFound = false;
			if (!bFound)
			{
				MapApplication.Current.Status("OK", false);
				MapApplication.Current.Message("No feature found.");
				return;
			}
			Geometry g = null;
			foreach (IdentifyResult result in e.IdentifyResults)
			{
				g = result.Feature.Geometry;
				break;
			}
			if (g == null)
			{
				MapApplication.Current.Status("OK", false);
				MapApplication.Current.Message("No network feature found.");
				return;
			}
			SpatialReference sr = g.SpatialReference;

			// Get vertex closest to identify point

			List<MapPoint> ptList = new List<MapPoint>();
			Polyline pline = (Polyline)g;
			MapPoint ptResult = pline.Paths[0][0];
			double dMin = g.Extent.Width * g.Extent.Height;
			foreach (PointCollection pc in pline.Paths)
			{
				foreach (MapPoint pt in pc)
				{
					double dX = (_ptIdentify.X - pt.X);
					double dY = (_ptIdentify.Y - pt.Y);
					double d = System.Math.Sqrt((dX * dX) + (dY * dY));
					if (d < dMin)
					{
						ptResult = pt;
						dMin = d;
					}
				}
			}
			ptResult.SpatialReference = sr;
			Graphic gr = new Graphic();
			gr.Geometry = ptResult;

			// Call geoprocessing service

			MapApplication.Current.Status("Calling service...", true);
			List<GPParameter> parameters = new List<GPParameter>();
			GPString gpsNetwork = new GPString("Workspace", _sWorkspace);
			parameters.Add(gpsNetwork);
			GPFeatureRecordSetLayer gplFlag = new GPFeatureRecordSetLayer("Flags");
			gplFlag.FeatureSet.Features.Add(gr);
			gplFlag.FeatureSet.SpatialReference = sr;
			parameters.Add(gplFlag);
			string sURL = _localTraceService.UrlGeoprocessingService;
			sURL += "/Protective+Device+Trace";
			Geoprocessor gp = new Geoprocessor(sURL);
			gp.ExecuteCompleted += new EventHandler<GPExecuteCompleteEventArgs>(gp_ExecuteCompleted);
			gp.Failed += new EventHandler<TaskFailedEventArgs>(gp_Failed);
			try
			{
				gp.ExecuteAsync(parameters);
			}
			catch (Exception ex)
			{
				string sMsg = "Error calling service:\n" + ex.Message + "\n" + ex.StackTrace;
				MapApplication.Current.Error(sMsg);
			}
		
		}

		private void it_Failed(object sender, TaskFailedEventArgs e)
		{
			MapApplication.Current.Error("Protective Device Trace failed:\n" + e.Error.Message);
		}

		private void gp_ExecuteCompleted(object sender, GPExecuteCompleteEventArgs e)
		{

			// Populate results layer

			ResultsLayer.Graphics.Clear();
			int iSwitchBanks = 0;
			int iFuseBanks = 0;
			int iRecloserBanks = 0;
			Envelope env = null;
			foreach (GPParameter gp in e.Results.OutParameters)
			{

				if (!(gp is GPFeatureRecordSetLayer))
					continue;
				GPFeatureRecordSetLayer gpFeats = (GPFeatureRecordSetLayer)gp;
				if (gpFeats.FeatureSet.Features == null)
					continue;
				string sName = gp.Name;
				string sMapTip = "";
				foreach (Graphic gr in gpFeats.FeatureSet.Features)
				{
					if (env == null)
						env = gr.Geometry.Extent;
					else
						env = env.Union(gr.Geometry.Extent);
					if (gr.Geometry is Polyline)
					{
						SimpleLineSymbol sls = new SimpleLineSymbol();
						sls.Color = System.Windows.Media.Brushes.Red;
						sls.Style = SimpleLineSymbol.LineStyle.Solid;
						sls.Width = 2;
						gr.Symbol = sls;
						switch (sName)
						{
							case "Overhead_Conductor":
								sMapTip = this.MakeMapTip("Overhead", gr, "FEEDERID");
								break;
							case "Underground_Conductor":
								sMapTip = this.MakeMapTip("Underground", gr, "FEEDERID");
								break;
						}
					}
					else if (gr.Geometry is MapPoint)
					{
						SimpleMarkerSymbol sms = new SimpleMarkerSymbol();
						sms.Color = System.Windows.Media.Brushes.Red;
						sms.Size = 16;
						switch (sName)
						{
							case "Switch_Banks":
								sMapTip = this.MakeMapTip("Switch", gr, "STRUCTURENUMBER");
								sms.Style = SimpleMarkerSymbol.SimpleMarkerStyle.Square;
								iSwitchBanks += 1;
								break;
							case "Fuse_Banks":
								sMapTip = this.MakeMapTip("Fuse", gr, "STRUCTURENUMBER");
								sms.Style = SimpleMarkerSymbol.SimpleMarkerStyle.Diamond;
								iFuseBanks += 1;
								break;
							case "Recloser_Banks":
								sMapTip = this.MakeMapTip("Recloser", gr, "STRUCTURENUMBER");
								sms.Style = SimpleMarkerSymbol.SimpleMarkerStyle.Circle;
								iRecloserBanks += 1;
								break;
						}
						gr.Symbol = sms;
					}
					else
						continue;
					if (sMapTip != "")
					{
						TextBlock tb = new TextBlock();
						tb.Text = sMapTip;
						tb.Background = System.Windows.Media.Brushes.LightYellow;
						gr.MapTip = tb;
						ResultsLayer.Graphics.Add(gr);
					}

				}
			}

			// Zoom or pan to results

			if (env != null)
			{
				env = env.Expand(1.1);
				Envelope envMap = _Map.Extent;
				if ((env.Width < envMap.Width) && (env.Height < envMap.Height))
					_Map.PanTo(env);
				else
					_Map.ZoomTo(env);
			}

			// Done

			MapApplication.Current.Status("OK", false);
			string sMsg = "Protective Devices Found:";
			sMsg += "\n  Switch Banks: " + iSwitchBanks.ToString();
			sMsg += "\n  Fuse Banks: " + iFuseBanks.ToString();
			sMsg += "\n  Recloser Banks: " + iRecloserBanks.ToString();
			MapApplication.Current.Message(sMsg);

		}

		private void gp_Failed(object sender, TaskFailedEventArgs e)
		{
			string sErr = "Protective Device Trace Execute Error:\n" + e.Error.Message;
			ServiceException se = (ServiceException)e.Error;
			if (se.Details.Count > 0)
				sErr += "\n" + se.Details[0];
			MapApplication.Current.Error(sErr);
		}

		private string MakeMapTip(string sType, Graphic gr, string sFieldName)
		{
			string sMapTip = sType + ": ";
			if (gr.Attributes.ContainsKey(sFieldName))
			{
				object val = gr.Attributes[sFieldName];
				if (val != null)
					sMapTip += val.ToString();
			}
			return sMapTip;
		}
		#endregion


	}


}
