﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Controls;
using System.Windows.Input;

using FieldApp.Extensibility;

namespace EditAddin
{

	// Convenient base class for edit commands
	#region EditButton
	public abstract class EditButton : IMapButton
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
		protected string _sButtonContent;
		protected string _sIconPath;
		protected string _sToolTip;

		#region Overridable Methods
		abstract protected void EdExecute();
		#endregion

		#region IMapButton Implementation
		public object ButtonContent
		{
			get
			{
				if (_sButtonContent != null)
					return _sButtonContent;
				else if (_sIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sIconPath);
				return null;
			}
		}

		public bool IsEnabled
		{
			get
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					EditBehavior eb = EditBehavior.Current;
					if (!eb.IsInitialized)
					{
						eb.Initialized += new EventHandler<EventArgs>(eb_Initialized);
						return false;
					}
					_bMapEnabled = true;
				}
				return _bMapEnabled;
			}
		}

		public string ToolTip
		{
			get { return _sToolTip; }
		}

		public void OnClick()
		{
			this.EdExecute();
		}

		public event EventHandler<EventArgs> IsEnabledChanged;
		#endregion

		void eb_Initialized(object sender, EventArgs e)
		{
			_bMapEnabled = true;
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

	}
	#endregion

	// Convenient base class for edit tools
	#region EditTool
	public abstract class EditTool : IMapTool
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
		protected string _sButtonContent;
		protected string _sIconPath;
		protected string _sToolTip;
		protected string _sCursorPath;

		#region Overridable Methods
		abstract protected void EdInitialized();
		abstract protected void EdActivateTool();
		abstract protected void EdDeactivateTool();
		#endregion

		#region IMapTool Implementation
		public object ButtonContent
		{
			get
			{
				if (_sButtonContent != null)
					return _sButtonContent;
				else if (_sIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sIconPath);
				return null;
			}
		}

		public Cursor MouseCursor
		{
			get
			{
				if (_sCursorPath == null)
					return Cursors.Cross;
				return AddinHelper.GetCursor(_MyAssembly, _sCursorPath);
			}
		}

		public bool IsEnabled
		{
			get
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					EditBehavior eb = EditBehavior.Current;
					if (!eb.IsInitialized)
					{
						eb.Initialized += new EventHandler<EventArgs>(eb_Initialized);
						return false;
					}
					_bMapEnabled = true;
				}
				return _bMapEnabled;
			}
		}

		public string ToolTip
		{
			get { return _sToolTip; }
		}

		public void Activate()
		{
			this.EdActivateTool();
		}

		public void Deactivate()
		{
			this.EdDeactivateTool();
		}

		public event EventHandler<EventArgs> IsEnabledChanged;
		#endregion

		void eb_Initialized(object sender, EventArgs e)
		{
			this.EdInitialized();
			_bMapEnabled = true;
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

	}
	#endregion

	#region EditSplitTool
	public abstract class EditSplitTool : IMapTool, IMapSplitButton
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
		protected string _sButtonContent;
		protected string _sIconPath;
		protected string _sDropIconPath;
		protected string _sToolTip;
		protected string _sCursorPath;

		#region Overridable Methods
		abstract protected void EdInitialized();
		abstract protected void EdActivateTool();
		abstract protected void EdDeactivateTool();
		abstract protected void EdOnItemSelected(IMapMenuItem item);
		#endregion

		#region IMapTool Implementation
		public object ButtonContent
		{
			get
			{
				if (_sButtonContent != null)
					return _sButtonContent;
				else if (_sIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sIconPath);
				return null;
			}
		}

		public Cursor MouseCursor
		{
			get
			{
				if (_sCursorPath == null)
					return Cursors.Cross;
				return AddinHelper.GetCursor(_MyAssembly, _sCursorPath);
			}
		}

		public bool IsEnabled
		{
			get
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					EditBehavior eb = EditBehavior.Current;
					if (!eb.IsInitialized)
					{
						eb.Initialized += new EventHandler<EventArgs>(eb_Initialized);
						return false;
					}
					_bMapEnabled = true;
				}
				return _bMapEnabled;
			}
		}

		public string ToolTip
		{
			get { return _sToolTip; }
		}

		public void Activate()
		{
			this.EdActivateTool();
		}

		public void Deactivate()
		{
			this.EdDeactivateTool();
		}

		public event EventHandler<EventArgs> IsEnabledChanged;
		#endregion

		#region IMapSplitButton implementation
		public object DropButtonContent
		{
			get
			{
				if (_sDropIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sDropIconPath);
				return null;
			}
		}

		public void OnItemSelected(IMapMenuItem item)
		{
			if (!_bMapEnabled)
				return;
			this.EdOnItemSelected(item);
			this.SelectionChanged(this, new EventArgs());
		}

		public event EventHandler<EventArgs> SelectionChanged;
		#endregion

		void eb_Initialized(object sender, EventArgs e)
		{
			this.EdInitialized();
			_bMapEnabled = true;
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

	}
	#endregion

	#region EditMenuItem
	public abstract class EditMenuItem : IMapMenuItem
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
		protected string _sLabel;
		protected string _sIconPath;
		protected bool _bIsCheckable;
		protected bool _bIsChecked;

		#region Overridable Methods
		abstract protected void EdExecute();
		#endregion

		#region IMapMenuItem implementation
		public Image Icon
		{
			get
			{
				if (_sIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sIconPath);
				return null;
			}
		}

		public bool IsCheckable
		{
			get { return _bIsCheckable; }
		}

		public bool IsChecked
		{
			get { return _bIsChecked; }
		}

		public bool IsEnabled
		{
			get
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					EditBehavior eb = EditBehavior.Current;
					if (!eb.IsInitialized)
					{
						eb.Initialized += new EventHandler<EventArgs>(eb_Initialized);
						return false;
					}
					_bMapEnabled = true;
				}
				return _bMapEnabled;
			}
		}

		public event EventHandler<EventArgs> IsEnabledChanged;

		public string Label
		{
			get { return _sLabel; }
		}

		public void OnClick()
		{
			this.EdExecute();
		}
		#endregion

		void eb_Initialized(object sender, EventArgs e)
		{
			_bMapEnabled = true;
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

	}
	#endregion

}
