﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.Linq;

using FieldApp.Extensibility;

namespace EditAddin
{

	// Uses 10.0 feature service functionality (not replica-aware)
	// ASSUMPTIONS:
	// 1) Records created or updated in the field are never deleted on the server
	// 2) Geometry types to be synchronized are 2D Points, Polylines, or Polygons
	// 3) Classes to be synchronized must have both IDSTAMP (string) and TIMESTAMP (datetime) fields
	//    [GLOBALID cannot be used because a feature service cannot add a feature with a given global id]
	// 4) Relationships between classes to be synchronized are through the IDSTAMP field

	// A feature is an XElement object consisting of 
	// the XML equivalent of the JSON feature specification

	public struct UpdateSet
	{
		public Dictionary<string, string> FieldInfo;
		public List<XElement> AddFeatureList;
		public List<XElement> UpdateFeatureList;
	}

	public struct ActionSet
	{
		public List<string> AddIDList;
		public List<string> UpdateIDList;
	}

	public interface ISynchronize
	{
		void SetStatus(IStatus pStatus);
		// SetStatus provides an optional way to send status updates to a GUI
		int GetUpdates(DateTime SyncDate, ISynchronize pTarget);
		// Used to get updates from server feature service
		// If SyncDate == DateTime.MinVal, all features are returned
		int GetUpdates(Dictionary<string, ActionSet> Actions, ISynchronize pTarget);
		// Used to get updates from local feature service
		int ProcessFeatures(Dictionary<string, UpdateSet> Features);
	}

	public class clsFieldHelper
	{

		private string _sLogFilePath;
		private List<string> _LayerNames;
		private ISynchronize _pLocalSync;
		private ISynchronize _pServerSync;
		private IStatus _pStatus;

		#region Public Methods
		public void Initialize(string sServiceUrl, string sLocalServiceUrl, List<string> LayerNames, string sLogFilePath, IStatus pStatus)
		{
			_sLogFilePath = sLogFilePath;
			_LayerNames = LayerNames;
			_pStatus = pStatus;
			_pLocalSync = new clsSyncFeatureService(true, sLocalServiceUrl, _LayerNames);
			_pLocalSync.SetStatus(_pStatus);
			_pServerSync = new clsSyncFeatureService(false, sServiceUrl, _LayerNames);
			_pServerSync.SetStatus(_pStatus);
		}

		public bool ProcessLocalFeature(string sLayerName, XElement xFeat, bool bAdd)
		{
			Dictionary<string, UpdateSet> d = new Dictionary<string, UpdateSet>();
			UpdateSet updateSet = new UpdateSet();
			if (bAdd)
			{
				updateSet.AddFeatureList = new List<XElement>();
				updateSet.AddFeatureList.Add(xFeat);
			}
			else
			{
				updateSet.UpdateFeatureList = new List<XElement>();
				updateSet.UpdateFeatureList.Add(xFeat);
			}
			d[sLayerName] = updateSet;
			bool bSuccess = (_pLocalSync.ProcessFeatures(d) > 0);
			if (bSuccess)
			{
				this.UpdateLog(sLayerName, xFeat, bAdd);
				if (bAdd)
					this.Message(sLayerName + " added.");
				else
					this.Message(sLayerName + " updated.");
			}
			else
				this.Error("Could not add or update " + sLayerName + ".");
			return bSuccess;
		}

		public void Synchronize()
		{
			this.Status("Synchronizing...", true);
			int iTotalSent = this.SendUpdates();
			if (iTotalSent < 0)
				return;
			DateTime dtSync = this.GetSyncDate(false); // Do not update until after success
			int iTotalReceived = _pServerSync.GetUpdates(dtSync, _pLocalSync);
			if (iTotalReceived < 0)
				return;
			dtSync = this.GetSyncDate(true);
			string sMsg = "Features sent: " + iTotalSent.ToString();
			sMsg += "\nFeatures received: " + iTotalReceived.ToString();
			this.Status("OK", false);
			this.Message(sMsg);
		}
		#endregion

		#region Private Methods
		private void Status(string sMsg)
		{
			if (_pStatus == null)
				return;
			_pStatus.Status(sMsg);
		}

		private void Status(string sMsg, bool bBusy)
		{
			if (_pStatus == null)
				return;
			_pStatus.Status(sMsg, bBusy);
		}

		private void Message(string sMsg)
		{
			if (_pStatus == null)
				return;
			_pStatus.Message(sMsg);
		}

		private void Error(string sErr)
		{
			if (_pStatus == null)
				return;
			_pStatus.Error(sErr);
		}

		private DateTime GetSyncDate(bool bUpdate)
		{

			// Open log and get sync node

			XDocument xEditLog = this.OpenEditLog();
			XElement xSync = this.FindSyncDateElement(xEditLog);

			// Get sync date

			DateTime dtSync = DateTime.MinValue;
			string sDate = this.GetAttribute(xSync, "date");
			if (sDate != null)
				dtSync = Convert.ToDateTime(sDate);
			if (!bUpdate)
				return dtSync;

			// Update sync date

			DateTime dtNow = DateTime.Now;
			xSync.SetAttributeValue("date", dtNow.ToString());

			// Save updated log

			xEditLog.Save(_sLogFilePath);
			return dtSync;

		}

		private int SendUpdates()
		{

			Dictionary<string, ActionSet> d = new Dictionary<string, ActionSet>();
			XDocument xEditLog = this.OpenEditLog();
			XElement xRoot = xEditLog.Root;
			List<XElement> ChildList = new List<XElement>();
			foreach (XElement xParent in xRoot.Elements())
			{

				// Get Layer type
				
				string sElementName = xParent.Name.LocalName;
				if (sElementName != "layer")
					continue;
				string sLayerName = this.GetAttribute(xParent, "name");

				// Build actions

				ActionSet actionSet = new ActionSet();
				actionSet.AddIDList = new List<string>();
				actionSet.UpdateIDList = new List<string>();
				int i = 0;
				foreach (XElement xChild in xParent.Elements())
				{
					string sID = this.GetAttribute(xChild, "id");
					string sAction = this.GetAttribute(xChild, "action");
					if (sAction == "add")
						actionSet.AddIDList.Add(sID);
					else
						actionSet.UpdateIDList.Add(sID);
					ChildList.Add(xChild);
					i++;
				}
				if (i == 0)
					continue;
				d[sLayerName] = actionSet;

			}

			// Send updates

			int iTotalSent = _pLocalSync.GetUpdates(d, _pServerSync);
			if (iTotalSent < 0)
				return -1;

			// Remove actions from log

			foreach (XElement xChild in ChildList)
				xChild.Remove();

			// Save updated log

			xEditLog.Save(_sLogFilePath);
			return iTotalSent;

		}

		private XDocument OpenEditLog()
		{
			XDocument xEditLog = new XDocument();
			if (File.Exists(_sLogFilePath))
			{
				XmlReader xr = XmlReader.Create(_sLogFilePath);
				xEditLog = XDocument.Load(xr);
				xr.Close();
			}
			else
			{
				XElement xElement = new XElement("FieldEdits");
				xEditLog.Add(xElement);
			}
			return xEditLog;
		}

		private XElement FindSyncDateElement(XDocument xEditLog)
		{

			// [The synchronization date node is named after SyncConstants.SyncName]

			XElement xRoot = xEditLog.Root;
			foreach (XElement xParent in xRoot.Elements())
			{
				if (xParent.Name.LocalName == "Synchronize")
					return xParent;
			}
			XElement xNewElement = new XElement("Synchronize");
			xRoot.Add(xNewElement);
			return xNewElement;
		}

		private XElement FindLayerElement(XDocument xEditLog, string sLayerName)
		{

			// [Layer nodes are named "layer", have the attribute "name", and contain feature nodes]

			XElement xRoot = xEditLog.Root;
			foreach (XElement xParent in xRoot.Elements())
			{
				if (xParent.Name.LocalName != "layer")
					continue; // This should never happen
				if (this.GetAttribute(xParent, "name") == sLayerName)
					return xParent;
			}
			XElement xNewElement = new XElement("layer");
			xNewElement.SetAttributeValue("name", sLayerName);
			xRoot.Add(xNewElement);
			return xNewElement;
		}

		private string GetAttribute(XElement xElement, string sName)
		{
			foreach (XAttribute xAtt in xElement.Attributes())
			{
				if (xAtt.Name.LocalName == sName)
					return xAtt.Value;
			}
			return null;
		}

		private void UpdateLog(string sLayerName, XElement xFeat, bool bNew)
		{

			// Get ID

			string sFeatID = "";
			foreach (XElement xChild in xFeat.Elements())
			{
				string sName = xChild.Name.LocalName;
				if (sName != "attributes")
					continue;
				foreach (XElement xAttr in xChild.Elements())
				{
					string sAttr = xAttr.Name.LocalName.ToUpper();
					if (sAttr != "IDSTAMP")
						continue;
					sFeatID = xAttr.Value;
				}
				break;
			}
			if (sFeatID == "")
				return;
			string sAction;
			if (bNew)
				sAction = "add";
			else
				sAction = "update";

			// Update log

			XDocument xEditLog = OpenEditLog();

			// Get parent node for feature type

			XElement xParent = this.FindLayerElement(xEditLog, sLayerName);

			// [Feature nodes are named "feature" and have attributes "action" and "id"]
			// [action can have the value "add" or "update"]

			// If a feature is updated, check if an entry already exists

			if (!bNew)
			{

				XElement xPrevious = null;
				foreach (XElement xElement in xParent.Elements())
				{
					string sID = this.GetAttribute(xElement, "id");
					if (sID == sFeatID)
					{
						xPrevious = xElement;
						break;
					}
				}
				if (xPrevious != null)
					return;
			}

			// Add feature

			XElement xFeatElement = new XElement("feature");
			xFeatElement.SetAttributeValue("action", sAction);
			xFeatElement.SetAttributeValue("id", sFeatID);
			xParent.Add(xFeatElement);

			// Save updated log

			xEditLog.Save(_sLogFilePath);

		}
		#endregion

	}
}
