﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using System.Xml;
using System.Xml.Linq;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

namespace MapHelpers
{

	public static class clsHelperCollection
	{

		public static clsCommonHelper CommonHelper;
		public static clsGraphicHelper GraphicHelper;
		public static clsGeometryHelper GeometryHelper;
		public static esriUnits MapUnits = esriUnits.esriUnknownUnits;
		public static void InitializeHelpers()
		{
			CommonHelper = new clsCommonHelper();
			GraphicHelper = new clsGraphicHelper();
			GeometryHelper = new clsGeometryHelper();
		}

	}

	public class clsCommonHelper
	{


		#region Public Methods and Properties
		public clsCommonHelper()
		{
		}

		public esriUnits GetMapUnits(Map MyMap)
		{

			// Check if map is web mercator

			SpatialReference sr = MyMap.SpatialReference;
			if (sr != null)
			{
				if ((sr.WKID == 102100) || (sr.WKID == 3857))
					return esriUnits.esriMeters;
			}

			// Assume that the first layer defines units

			Layer l = MyMap.Layers[0];
			string sLayerUnits = "";
			if (l is ArcGISTiledMapServiceLayer)
			{
				ArcGISTiledMapServiceLayer tiledLayer = (ArcGISTiledMapServiceLayer)l;
				sLayerUnits = tiledLayer.Units;
			}
			else if (l is ArcGISDynamicMapServiceLayer)
			{
				ArcGISDynamicMapServiceLayer dynLayer = (ArcGISDynamicMapServiceLayer)l;
				sLayerUnits = dynLayer.Units;
			}
			return this.StringToUnits(sLayerUnits);

		}

		public void ZoomTo(Map MyMap, ESRI.ArcGIS.Client.Geometry.Geometry g)
		{

			// Improves on Map.ZoomTo

			if (g == null)
				return;
			Envelope env = g.Extent;
			if ((env.Width > 0) && (env.Height > 0))
			{
				Envelope envNew = env.Expand(1.1);
				MyMap.ZoomTo(envNew);
				return;
			}
			MapPoint pt = env.GetCenter();
			double dW = env.Width * 1.1 / 2;
			double dH = env.Height * 1.1 / 2;
			if ((dW == 0) && (dH == 0))
			{
				double dSF = 1000 / MyMap.Scale;
				dW = MyMap.Extent.Width * dSF / 2;
				dH = MyMap.Extent.Height * dSF / 2;
			}
			else if (dW == 0)
				dW = dH;
			else if (dH == 0)
				dH = dW;
			env = new Envelope(pt.X - dW, pt.Y - dH, pt.X + dW, pt.Y + dH);
			MyMap.ZoomTo(env);

		}

		public bool IsSublayerVisible(ArcGISDynamicMapServiceLayer dynLayer, int iSublayerID, double dScale)
		{

			bool bVisible = true;
			int[] VisLayers = dynLayer.VisibleLayers;
			if (VisLayers != null)
			{
				bVisible = false;
				foreach (int iVis in VisLayers)
				{
					if (iVis != iSublayerID)
						continue;
					bVisible = true;
					break;
				}
			}
			if (!bVisible)
				return false;
			double dMin = 0;
			double dMax = 0;
			foreach (LayerInfo li in dynLayer.Layers)
			{
				if (li.ID != iSublayerID)
					continue;
				bVisible = li.DefaultVisibility;
				dMin = li.MinScale;
				dMax = li.MaxScale;
				break;
			}
			if (!bVisible)
				return false;
			if (dScale <= 0)
				return true;
			if (((dMin != 0) && (dScale > dMin)) || ((dMax != 0) && (dScale < dMax)))
				return false;
			return true;

		}

		public string FindLayerURL(Map TheMap, string sLayerName, int iInstance = 1)
		{
			string sURL = "";
			LayerCollection lc = TheMap.Layers;
			int iCount = 0;
			foreach (Layer l in lc)
			{
				if (!(l is ArcGISDynamicMapServiceLayer))
					continue;
				ArcGISDynamicMapServiceLayer dl = (ArcGISDynamicMapServiceLayer)l;
				string sBaseURL = dl.Url;
				LayerInfo[] LayerInfos = dl.Layers;
				foreach (LayerInfo li in LayerInfos)
				{
					if (li.Name != sLayerName)
						continue;
					iCount++;
					if (iCount < iInstance)
						continue;
					sURL = sBaseURL + "/" + li.ID.ToString();
					return sURL;
				}
			}
			return sURL;
		}

		public string FindTableURL(Map TheMap, string sTableName)
		{
			string sURL = "";
			LayerCollection lc = TheMap.Layers;
			foreach (Layer l in lc)
			{
				if (!(l is ArcGISDynamicMapServiceLayer))
					continue;
				ArcGISDynamicMapServiceLayer dl = (ArcGISDynamicMapServiceLayer)l;
				string sBaseURL = dl.Url;
				Table[] TableList = dl.Tables;
				if (TableList == null)
					continue;
				foreach (Table t in TableList)
				{
					if (t.Name != sTableName)
						continue;
					sURL = sBaseURL + "/" + t.ID.ToString();
					return sURL;
				}
			}
			return sURL;
		}

		public string FindLayerOrTableURL(ArcGISDynamicMapServiceLayer dynLayer, string sName)
		{
			string sURL = "";
			LayerInfo[] LayerInfos = dynLayer.Layers;
			foreach (LayerInfo li in LayerInfos)
			{
				if (li.Name != sName)
					continue;
				sURL = dynLayer.Url + "/" + li.ID.ToString();
				return sURL;
			}
			Table[] TableList = dynLayer.Tables;
			foreach (Table t in TableList)
			{
				if (t.Name != sName)
					continue;
				sURL = dynLayer.Url + "/" + t.ID.ToString();
				return sURL;
			}
			return sURL;
		}
		#endregion

		#region Private Methods
		private esriUnits StringToUnits(string sLayerUnits)
		{

			switch (sLayerUnits)
			{
				case "esriCentimeters":
					return esriUnits.esriCentimeters;
				case "esriDecimalDegrees":
					return esriUnits.esriDecimalDegrees;
				case "esriDecimeters":
					return esriUnits.esriDecimeters;
				case "esriFeet":
					return esriUnits.esriFeet;
				case "esriInches":
					return esriUnits.esriInches;
				case "esriKilometers":
					return esriUnits.esriKilometers;
				case "esriMeters":
					return esriUnits.esriMeters;
				case "esriMiles":
					return esriUnits.esriMiles;
				case "esriMillimeters":
					return esriUnits.esriMillimeters;
				case "esriNauticalMiles":
					return esriUnits.esriNauticalMiles;
				case "esriPoints":
					return esriUnits.esriPoints;
				case "esriYards":
					return esriUnits.esriYards;
			}
			return esriUnits.esriUnknownUnits;

		}

		#endregion

	}
}
