﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

namespace MapHelpers
{

	public class GeometryResultArgs : EventArgs
	{
		private Geometry _geom;
		private object _userToken;
		public GeometryResultArgs()
		{
		}
		public Geometry Geometry
		{
			set { _geom = value; }
			get { return _geom; }
		}
		public object UserState
		{
			set { _userToken = value; }
			get { return _userToken; }
		}
	}

	public class DifferenceResultArgs : EventArgs
	{
		private IList<Graphic> _grList;
		private object _userToken;
		public DifferenceResultArgs()
		{
		}
		public IList<Graphic> Result
		{
			set { _grList = value; }
			get { return _grList; }
		}
		public object UserState
		{
			set { _userToken = value; }
			get { return _userToken; }
		}
	}

	public class HelperFailedArgs : EventArgs
	{
		private Exception _e;
		private object _userToken;
		public HelperFailedArgs()
		{
		}
		public Exception Error
		{
			set { _e = value; }
			get { return _e; }
		}
		public object UserState
		{
			set { _userToken = value; }
			get { return _userToken; }
		}
	}

	public class clsGeometryHelper
	{

		public bool Initialized { get; private set; }

		private class LineSegment
		{
			public MapPoint From;
			public MapPoint To;
			public LineSegment(MapPoint ptFrom, MapPoint ptTo)
			{
				From = ptFrom;
				To = ptTo;
			}
			public double Length
			{
				get { return clsGeometryHelper.Distance(From, To); }
			}
			public MapPoint Along(double dDistance, bool bAsRatio)
			{
				double d = dDistance;
				if (!bAsRatio)
					d /= clsGeometryHelper.Distance(From, To);
				double dX = From.X + ((To.X - From.X) * d);
				double dY = From.Y + ((To.Y - From.Y) * d);
				return new MapPoint(dX, dY);
			}
		}

		private string _sUrl;
		private GeometryService _geomService = null;

		#region Public Methods and Events
		public clsGeometryHelper()
		{
			Initialized = false;
		}

		public void Initialize(string sGeometryServiceUrl)
		{
			_sUrl = sGeometryServiceUrl;
			_geomService = new GeometryService();
			_geomService.Url = sGeometryServiceUrl;
			_geomService.LabelPointsCompleted += new EventHandler<GraphicsEventArgs>(_geomService_LabelPointsCompleted);
			_geomService.ProjectCompleted += new EventHandler<GraphicsEventArgs>(_geomService_ProjectCompleted);
			_geomService.BufferCompleted += new EventHandler<GraphicsEventArgs>(_geomService_BufferCompleted);
			_geomService.IntersectCompleted += new EventHandler<GraphicsEventArgs>(_geomService_IntersectCompleted);
			_geomService.DifferenceCompleted += new EventHandler<GraphicsEventArgs>(_geomService_DifferenceCompleted);
			_geomService.Failed += new EventHandler<TaskFailedEventArgs>(_geomService_Failed);
			Initialized = true;
		}

		public string Url { get { return _sUrl; } }

		public event EventHandler<GeometryResultArgs> GetCenterCompleted;
		public event EventHandler<GeometryResultArgs> ProjectPointCompleted;
		public event EventHandler<GeometryResultArgs> BufferCompleted;
		public event EventHandler<GeometryResultArgs> IntersectCompleted;
		public event EventHandler<DifferenceResultArgs> DifferenceCompleted;
		public event EventHandler<HelperFailedArgs> Failed;

		#region Asynchronous Service Methods (Silverlight Compatible)
		public void GetCenterAsync(Geometry g, object userToken = null)
		{
			MapPoint pt = null;
			if (g is Polyline)
				pt = this.Along((Polyline)g, 0.5, true);
			else if (g is Polygon)
			{

				// Get label point from geometry service

				if (_geomService == null)
				{
					this.IntFailed("Geometry service not initialized", userToken);
					return;
				}
				Graphic gr = new Graphic();
				gr.Geometry = g;
				IList<Graphic> grList = new List<Graphic>();
				grList.Add(gr);
				_geomService.LabelPointsAsync(grList, userToken);
				return;

			}
			else
				pt = g.Extent.GetCenter();
			GeometryResultArgs args = new GeometryResultArgs();
			args.Geometry = pt;
			args.UserState = userToken;
			this.GetCenterCompleted(this, args);
		}

		public void ProjectPointAsync(MapPoint ptMap, SpatialReference outSR, object userToken = null)
		{

			// Assume no datum transformation required

			if (_geomService == null)
			{
				this.IntFailed("Geometry service not initialized", userToken);
				return;
			}
			Graphic gr = new Graphic();
			gr.Geometry = ptMap;
			IList<Graphic> grList = new List<Graphic>();
			grList.Add(gr);
			_geomService.ProjectAsync(grList, outSR, userToken);

		}

		public void BufferAsync(Geometry g, double dBufferSize, object userToken = null)
		{
			if (_geomService == null)
			{
				this.IntFailed("Geometry service not initialized", userToken);
				return;
			}
			Graphic gr = new Graphic();
			gr.Geometry = g;
			BufferParameters bp = new BufferParameters();
			bp.Features.Add(gr);
			bp.Distances.Add(dBufferSize);
			_geomService.BufferAsync(bp, userToken);
		}

		public void IntersectionAsync(Geometry g1, Geometry g2, object userToken = null)
		{
			if (_geomService == null)
			{
				this.IntFailed("Geometry service not initialized", userToken);
				return;
			}
			Graphic gr1 = new Graphic();
			gr1.Geometry = g1;
			IList<Graphic> grList = new List<Graphic>();
			grList.Add(gr1);
			_geomService.IntersectAsync(grList, g2, userToken);
		}

		public void DifferenceAsync(Graphic grIn, Geometry g, object userToken = null)
		{
			if (_geomService == null)
			{
				this.IntFailed("Geometry service not initialized", userToken);
				return;
			}
			List<Graphic> grList = new List<Graphic>();
			grList.Add(grIn);
			_geomService.DifferenceAsync(grList, g, userToken);
		}
		#endregion

		#region Non-Service Methods
		public static double Distance(MapPoint pt1, MapPoint pt2)
		{
			double dX = pt2.X - pt1.X;
			double dY = pt2.Y - pt1.Y;
			return Math.Sqrt((dX * dX) + (dY * dY));
		}

		public double ConvertLinearUnit(double dVal, esriUnits inUnits, esriUnits outUnits)
		{
			double dNewVal = 0;
			if (inUnits == outUnits)
				return dVal;
			if ((inUnits == esriUnits.esriUnknownUnits) || (outUnits == esriUnits.esriUnknownUnits))
				return dNewVal;
			if ((inUnits == esriUnits.esriDecimalDegrees) || (outUnits == esriUnits.esriDecimalDegrees))
				return dNewVal;
			double dSF = 0.3048;
			if (this.LinearIsMetric(inUnits))
			{
				dNewVal = this.LinearMetricToMeters(dVal, inUnits);
				if (this.LinearIsMetric(outUnits))
					return this.LinearMetersToMetric(dNewVal, outUnits);
				dNewVal /= dSF;
				return this.LinearFeetToEnglish(dNewVal, outUnits);
			}
			dNewVal = this.LinearEnglishToFeet(dVal, inUnits);
			if (!this.LinearIsMetric(outUnits))
				return this.LinearFeetToEnglish(dNewVal, outUnits);
			dNewVal *= dSF;
			return this.LinearMetersToMetric(dNewVal, outUnits);
		}

		public double ConvertAreaUnit(double dVal, AreaUnit inUnits, AreaUnit outUnits)
		{
			double dNewVal = 0;
			if (inUnits == outUnits)
				return dVal;
			if ((inUnits == AreaUnit.Default) || (outUnits == AreaUnit.Default))
				return dNewVal;
			double dSF = 0.3048 * 0.3048;
			if (this.AreaIsMetric(inUnits))
			{
				dNewVal = this.AreaMetricToMeters(dVal, inUnits);
				if (this.AreaIsMetric(outUnits))
					return this.AreaMetersToMetric(dNewVal, outUnits);
				dNewVal /= dSF;
				return this.AreaFeetToEnglish(dNewVal, outUnits);
			}
			dNewVal = this.AreaEnglishToFeet(dVal, inUnits);
			if (!this.AreaIsMetric(outUnits))
				return this.AreaFeetToEnglish(dNewVal, outUnits);
			dNewVal *= dSF;
			return this.AreaMetersToMetric(dNewVal, outUnits);
		}

		public string ToDMS(double d, bool bLong)
		{
			int iD = (int)Math.Floor(Math.Abs(d));
			string sDegrees = iD.ToString() + "°";
			double dMS = (Math.Abs(d) - (double)iD) * 60;
			int iM = (int)Math.Floor(Math.Abs(dMS));
			sDegrees += iM.ToString() + "'";
			double dS = (dMS - iM) * 60;
			sDegrees += dS.ToString("N3") + '"';
			if (bLong)
			{
				if (d < 0)
					sDegrees += "W";
				else
					sDegrees += "E";
			}
			else
			{
				if (d < 0)
					sDegrees += "S";
				else
					sDegrees += "N";
			}
			return sDegrees;
		}

		public double FromDMS(string sDegrees)
		{
			double dDegrees = 0;
			string sEntry = sDegrees.ToUpper();
			bool bNeg = (sEntry.Contains("W") || sEntry.Contains("S"));
			char[] seps = { '°', '\'', '"', 'W', 'E', 'S', 'N', ' ' };
			string[] sTokens = sEntry.Split(seps);
			int i = 0;
			foreach (string sToken in sTokens)
			{
				double d = double.NaN;
				try
				{
					d = Convert.ToDouble(sToken);
				}
				catch
				{
				}
				if (d == double.NaN)
					return d;
				switch (i)
				{
					case 0:
						dDegrees += d;
						break;
					case 1:
						dDegrees += d / 60;
						break;
					case 2:
						dDegrees += d / 3600;
						break;
				}
				i++;
			}
			if (bNeg)
				dDegrees = -dDegrees;
			return dDegrees;
		}
		#endregion
		#endregion

		#region Event Handlers
		private void _geomService_LabelPointsCompleted(object sender, GraphicsEventArgs e)
		{
			IList<Graphic> resultList = e.Results;
			Graphic gr = resultList[0];
			GeometryResultArgs args = new GeometryResultArgs();
			args.Geometry = gr.Geometry;
			args.UserState = e.UserState;
			this.GetCenterCompleted(this, args);
		}

		private void _geomService_ProjectCompleted(object sender, GraphicsEventArgs e)
		{
			IList<Graphic> resultList = e.Results;
			Graphic gr = resultList[0];
			GeometryResultArgs args = new GeometryResultArgs();
			args.Geometry = gr.Geometry;
			args.UserState = e.UserState;
			this.ProjectPointCompleted(this, args);
		}

		private void _geomService_BufferCompleted(object sender, GraphicsEventArgs e)
		{
			IList<Graphic> resultList = e.Results;
			Graphic gr = resultList[0];
			GeometryResultArgs args = new GeometryResultArgs();
			args.Geometry = gr.Geometry;
			args.UserState = e.UserState;
			this.BufferCompleted(this, args);
		}

		private void _geomService_IntersectCompleted(object sender, GraphicsEventArgs e)
		{
			IList<Graphic> resultList = e.Results;
			Graphic gr = resultList[0];
			GeometryResultArgs args = new GeometryResultArgs();
			args.Geometry = gr.Geometry;
			args.UserState = e.UserState;
			this.IntersectCompleted(this, args);
		}

		private void _geomService_DifferenceCompleted(object sender, GraphicsEventArgs e)
		{
			IList<Graphic> resultList = e.Results;
			DifferenceResultArgs args = new DifferenceResultArgs();
			args.Result = resultList;
			args.UserState = e.UserState;
			this.DifferenceCompleted(this, args);
		}

		private void _geomService_Failed(object sender, TaskFailedEventArgs e)
		{
			HelperFailedArgs args = new HelperFailedArgs();
			args.Error = e.Error;
			args.UserState = e.UserState;
			this.Failed(this, args);
		}
		#endregion

		#region Private Methods
		private void IntFailed(string sErr, object userToken)
		{
			Exception ex = new Exception(sErr);
			HelperFailedArgs args = new HelperFailedArgs();
			args.Error = ex;
			args.UserState = userToken;
			this.Failed(this, args);
		}

		private MapPoint Along(Polyline Line, double dDistance, bool bAsRatio)
		{

			// Return a point representing a measurement along a polyline

			MapPoint ptAlong = null;
			try
			{
				List<LineSegment> SegList = new List<LineSegment>();
				List<double> LenList = new List<double>();
				double dTotalLen = 0;
				int iCount = 0;

				foreach (PointCollection pc in Line.Paths)
				{
					MapPoint ptPrev = null;
					foreach (MapPoint pt in pc)
					{
						if (ptPrev == null)
						{
							ptPrev = pt;
							continue;
						}
						LineSegment seg = new LineSegment(ptPrev, pt);
						double dLen = seg.Length;
						SegList.Add(seg);
						LenList.Add(dLen);
						dTotalLen += dLen;
						iCount++;
						ptPrev = pt;
					}
				}

				double dQueryDistance = dDistance;
				if (bAsRatio)
					dQueryDistance *= dTotalLen;
				dTotalLen = 0;
				for (int i = 0; i < iCount; i++)
				{
					double dLen = LenList[i];
					dTotalLen += dLen;
					if ((dTotalLen < dQueryDistance) && (i < iCount - 1))
						continue;
					double d = dLen - (dTotalLen - dQueryDistance);
					LineSegment seg = SegList[i];
					ptAlong = seg.Along(d, false);
					if (ptAlong != null)
						ptAlong.SpatialReference = Line.SpatialReference;
					break;
				}
			}
			catch
			{
			}
			return ptAlong;

		}

		private bool LinearIsMetric(esriUnits Units)
		{
			switch (Units)
			{
				case esriUnits.esriCentimeters:
				case esriUnits.esriDecimeters:
				case esriUnits.esriKilometers:
				case esriUnits.esriMeters:
				case esriUnits.esriMillimeters:
				case esriUnits.esriNauticalMiles:
					return true;
			}
			return false;
		}

		private double LinearMetricToMeters(double dVal, esriUnits inUnits)
		{
			double dNewVal = dVal;
			switch (inUnits)
			{
				case esriUnits.esriCentimeters:
					dNewVal /= 100;
					break;
				case esriUnits.esriDecimeters:
					dNewVal /= 10;
					break;
				case esriUnits.esriKilometers:
					dNewVal *= 1000;
					break;
				case esriUnits.esriMillimeters:
					dNewVal /= 1000;
					break;
				case esriUnits.esriNauticalMiles:
					dNewVal *= 1852;
					break;
			}
			return dNewVal;
		}

		private double LinearMetersToMetric(double dVal, esriUnits outUnits)
		{
			double dNewVal = dVal;
			switch (outUnits)
			{
				case esriUnits.esriCentimeters:
					dNewVal *= 100;
					break;
				case esriUnits.esriDecimeters:
					dNewVal *= 10;
					break;
				case esriUnits.esriKilometers:
					dNewVal /= 1000;
					break;
				case esriUnits.esriMillimeters:
					dNewVal *= 1000;
					break;
				case esriUnits.esriNauticalMiles:
					dNewVal /= 1852;
					break;
			}
			return dNewVal;
		}

		private double LinearEnglishToFeet(double dVal, esriUnits inUnits)
		{
			double dNewVal = dVal;
			switch (inUnits)
			{
				case esriUnits.esriInches:
					dNewVal /= 12;
					break;
				case esriUnits.esriMiles:
					dNewVal *= 5280;
					break;
				case esriUnits.esriPoints:
					dNewVal /= (12 * 72);
					break;
				case esriUnits.esriYards:
					dNewVal *= 3;
					break;
			}
			return dNewVal;
		}

		private double LinearFeetToEnglish(double dVal, esriUnits outUnits)
		{
			double dNewVal = dVal;
			switch (outUnits)
			{
				case esriUnits.esriInches:
					dNewVal *= 12;
					break;
				case esriUnits.esriMiles:
					dNewVal /= 5280;
					break;
				case esriUnits.esriPoints:
					dNewVal *= (12 * 72);
					break;
				case esriUnits.esriYards:
					dNewVal /= 3;
					break;
			}
			return dNewVal;
		}

		private bool AreaIsMetric(AreaUnit Units)
		{
			switch (Units)
			{
				case AreaUnit.Ares:
				case AreaUnit.Hectares:
				case AreaUnit.SquareCentimeters:
				case AreaUnit.SquareDecimeters:
				case AreaUnit.SquareKilometers:
				case AreaUnit.SquareMeters:
				case AreaUnit.SquareMillimeters:
					return true;
			}
			return false;
		}

		private double AreaMetricToMeters(double dVal, AreaUnit inUnits)
		{
			double dNewVal = dVal;
			switch (inUnits)
			{
				case AreaUnit.Ares:
					dNewVal *= 10 * 10;
					break;
				case AreaUnit.Hectares:
					dNewVal *= 100 * 100;
					break;
				case AreaUnit.SquareCentimeters:
					dNewVal *= 0.01 * 0.01;
					break;
				case AreaUnit.SquareDecimeters:
					dNewVal *= 0.1 * 0.1;
					break;
				case AreaUnit.SquareKilometers:
					dNewVal *= 1000 * 1000;
					break;
				case AreaUnit.SquareMillimeters:
					dNewVal *= 0.001 * 0.001;
					break;
			}
			return dNewVal;
		}

		private double AreaMetersToMetric(double dVal, AreaUnit outUnits)
		{
			double dNewVal = dVal;
			switch (outUnits)
			{
				case AreaUnit.Ares:
					dNewVal /= 10 * 10;
					break;
				case AreaUnit.Hectares:
					dNewVal /= 100 * 100;
					break;
				case AreaUnit.SquareCentimeters:
					dNewVal /= 0.01 * 0.01;
					break;
				case AreaUnit.SquareDecimeters:
					dNewVal /= 0.1 * 0.1;
					break;
				case AreaUnit.SquareKilometers:
					dNewVal /= 1000 * 1000;
					break;
				case AreaUnit.SquareMillimeters:
					dNewVal /= 0.001 * 0.001;
					break;
			}
			return dNewVal;
		}

		private double AreaEnglishToFeet(double dVal, AreaUnit inUnits)
		{
			double dNewVal = dVal;
			switch (inUnits)
			{
				case AreaUnit.Acres:
					dNewVal *= 43560;
					break;
				case AreaUnit.SquareInches:
					dNewVal /= 12 * 12;
					break;
				case AreaUnit.SquareMiles:
					dNewVal *= 5280 * 5280;
					break;
				case AreaUnit.SquareYards:
					dNewVal *= 3 * 3;
					break;
			}
			return dNewVal;
		}

		private double AreaFeetToEnglish(double dVal, AreaUnit outUnits)
		{
			double dNewVal = dVal;
			switch (outUnits)
			{
				case AreaUnit.Acres:
					dNewVal /= 43560;
					break;
				case AreaUnit.SquareInches:
					dNewVal *= 12 * 12;
					break;
				case AreaUnit.SquareMiles:
					dNewVal /= 5280 * 5280;
					break;
				case AreaUnit.SquareYards:
					dNewVal /= 3 * 3;
					break;
			}
			return dNewVal;
		}
		#endregion
	}
}
