﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Symbols;

namespace MapHelpers
{

	public enum HighlighterToolType
	{
		None,
		HighlightYellow,
		HighlightGreen,
		HighlightBlue,
		EraserEntire,
		EraserSmall,
		EraserMedium,
		EraserLarge
	}

	public class clsHighlighter
	{

		private struct EraseList
		{
			public List<Graphic> GraphicList;
			public int Index;
			public ESRI.ArcGIS.Client.Geometry.Geometry Geometry;
		}

		private HighlighterToolType _CurrentTool;
		private Map _Map;
		private GraphicsLayer _lyrHighlight;
		private clsGeometryHelper _gh;
		private Draw _Highlight;

		public void Initialize(Map MyMap, GraphicsLayer lyrHighlight)
		{
			_CurrentTool = HighlighterToolType.None;
			_Map = MyMap;
			_lyrHighlight = lyrHighlight;
			string sUrl = clsHelperCollection.GeometryHelper.Url;
			_gh = new clsGeometryHelper();
			_gh.Initialize(sUrl);
			_gh.BufferCompleted += new EventHandler<GeometryResultArgs>(_gh_BufferCompleted);
			_gh.IntersectCompleted += new EventHandler<GeometryResultArgs>(_gh_IntersectCompleted);
			_gh.DifferenceCompleted += new EventHandler<DifferenceResultArgs>(_gh_DifferenceCompleted);
			_gh.Failed += new EventHandler<HelperFailedArgs>(_gh_Failed);
			_Highlight = new Draw(_Map);
			_Highlight.DrawMode = DrawMode.Freehand;
			_Highlight.IsEnabled = false;
			_Highlight.DrawComplete += new EventHandler<DrawEventArgs>(_Highlight_DrawComplete);
		}

		public void SetTool(HighlighterToolType htType)
		{
			_CurrentTool = htType;
			DrawMode drMode = DrawMode.Freehand;
			bool bEnabled = true;
			switch (htType)
			{
				case HighlighterToolType.EraserEntire:
					drMode = DrawMode.Point;
					break;
				case HighlighterToolType.None:
					bEnabled = false;
					break;
				default:
					break;
			}
			_Highlight.DrawMode = drMode;
			if (drMode == DrawMode.Freehand)
				_Highlight.LineSymbol = this.GetLineSymbol();
			_Highlight.IsEnabled = bEnabled;
		}

		private void _Highlight_DrawComplete(object sender, DrawEventArgs e)
		{
			double dPixelSize = this.GetPixelSize();
			double dBufSize = dPixelSize * _Map.Resolution;
			switch (_CurrentTool)
			{
				case HighlighterToolType.EraserEntire:
					MapPoint pt = (MapPoint)e.Geometry;
					Envelope env = new Envelope(pt.X - dBufSize, pt.Y - dBufSize, pt.X + dBufSize, pt.Y + dBufSize);
					this.Erase(env, true);
					break;
				case HighlighterToolType.HighlightYellow:
				case HighlighterToolType.HighlightGreen:
				case HighlighterToolType.HighlightBlue:
					this.Highlight(e.Geometry);
					break;
				default:
					_gh.BufferAsync(e.Geometry, dBufSize);
					break;
			}
		}

		void _gh_BufferCompleted(object sender, GeometryResultArgs e)
		{
			ESRI.ArcGIS.Client.Geometry.Geometry gBuf = e.Geometry;
			this.Erase(gBuf, false);
		}

		private void _gh_IntersectCompleted(object sender, GeometryResultArgs e)
		{

			ESRI.ArcGIS.Client.Geometry.Geometry gInt = e.Geometry;
			EraseList eraseList = (EraseList)e.UserState;
			List<Graphic> grList = eraseList.GraphicList;
			int i = eraseList.Index;

			// If intersection is not null, remove graphic

			Graphic gr = grList[i];
			if (gInt.Extent != null)
				_lyrHighlight.Graphics.Remove(gr);
			
			// If last graphic is processed, refresh layer

			if (i == grList.Count - 1)
			{
				_lyrHighlight.Refresh();
				return;
			}

			// Process next graphic

			i++;
			gr = grList[i];
			eraseList.Index = i;
			_gh.IntersectionAsync(gr.Geometry, eraseList.Geometry, eraseList);

		}

		private void _gh_DifferenceCompleted(object sender, DifferenceResultArgs e)
		{

			IList<Graphic> grResultList = e.Result;
			EraseList eraseList = (EraseList)e.UserState;
			List<Graphic> grList = eraseList.GraphicList;
			int i = eraseList.Index;
			Graphic gr = grList[i];

			// Add results to layer

			foreach (Graphic grResult in grResultList)
			{
				grResult.Symbol = gr.Symbol;
				_lyrHighlight.Graphics.Add(grResult);
			}

			// If last graphic is processed, refresh layer

			if (i == grList.Count - 1)
			{
				_lyrHighlight.Refresh();
				return;
			}

			// Process next graphic

			i++;
			gr = grList[i];
			eraseList.Index = i;
			_gh.DifferenceAsync(gr, eraseList.Geometry, eraseList);

		}

		private void _gh_Failed(object sender, HelperFailedArgs e)
		{
			string sMsg = "Geometry operation failed:\n" + e.Error.Message;
			MessageBox.Show(sMsg, "Query Helper", MessageBoxButton.OK);
		}

		private void Highlight(ESRI.ArcGIS.Client.Geometry.Geometry g)
		{
			Graphic gr = new Graphic();
			gr.Geometry = g;
			gr.Symbol = _Highlight.LineSymbol;
			_lyrHighlight.Graphics.Add(gr);
			_lyrHighlight.Refresh();
		}

		private void Erase(ESRI.ArcGIS.Client.Geometry.Geometry g, bool bEntire)
		{

			// Build list of graphics to process

			List<Graphic> grList = new List<Graphic>();
			foreach (Graphic gr in _lyrHighlight.Graphics)
				grList.Add(gr);
			if (grList.Count == 0)
				return;
			EraseList eraseList = new EraseList();
			eraseList.GraphicList = grList;
			eraseList.Index = 0;
			eraseList.Geometry = g;
			Graphic grFirst = grList[0];
			if (bEntire)
				_gh.IntersectionAsync(grFirst.Geometry, g, eraseList);
			else
			{
				_lyrHighlight.Graphics.Clear();
				_gh.DifferenceAsync(grList[0], g, eraseList);
			}
		}

		private SimpleLineSymbol GetLineSymbol()
		{
			Color c;
			switch (_CurrentTool)
			{
				case HighlighterToolType.HighlightYellow:
					c = Colors.Yellow;
					c.A = 127;
					break;
				case HighlighterToolType.HighlightGreen:
					c = Colors.Lime; // Use Green for Silverlight
					c.A = 127;
					break;
				case HighlighterToolType.HighlightBlue:
					c = Colors.Cyan;
					c.A = 127;
					break;
				default:
					c = Colors.White;
					break;
			}
			return new SimpleLineSymbol(c, this.GetPixelSize());
		}

		private double GetPixelSize()
		{
			double dPixelSize = 6;
			switch (_CurrentTool)
			{
				case HighlighterToolType.EraserSmall:
					dPixelSize = 1;
					break;
				case HighlighterToolType.EraserMedium:
					dPixelSize = 4;
					break;
				case HighlighterToolType.EraserLarge:
					dPixelSize = 8;
					break;
			}
			return dPixelSize;
		}

	}
}
