﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.FeatureService;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

namespace MapHelpers
{

	public enum LayerSearchType
	{
		Topmost,
		Visible,
		All,
		Single
	}

	public delegate void IdentifyGeometryCallback(Geometry g);

	public class LayerSearchItem
	{

		public LayerSearchType SearchType = LayerSearchType.Topmost;
		public int LayerIndex = 0;
		public int SubLayerID = 0;
		public string SubLayerName = "";

		public override string ToString()
		{
			string sVal = "";
			switch (SearchType)
			{
				case LayerSearchType.Topmost:
					sVal = "<Top-most layer>";
					break;
				case LayerSearchType.Visible:
					sVal = "<Visible layers>";
					break;
				case LayerSearchType.All:
					sVal = "<All layers>";
					break;
				case LayerSearchType.Single:
					sVal = SubLayerName;
					break;
			}
			return sVal;
		}
	}

	public class SearchFeature
	{
		public string Value { get; set; }
		public bool IsExpanded { get; set; }
		public bool IsSelected { get; set; }
		public Graphic Feature { get; set; }
		public bool HasGeometry
		{
			get 
			{
				if (Feature == null)
					return false;
				return Feature.Geometry != null; 
			}
		}
		public int ObjectID { get; set; }
		public string FieldName { get; set; }
		public int LayerIndex { get; set; }
		public string LayerName { get; set; }
		private ObservableCollection<SearchFeature> _records = new ObservableCollection<SearchFeature>();
		public ObservableCollection<SearchFeature> RelatedRecords
		{
			get { return _records; }
		}
		public void AddRelatedRecord(SearchFeature Rec)
		{
			_records.Add(Rec);
		}
	}

	public struct QueryLayer
	{
		public int LayerIndex;
		public int SublayerID;
		public string SublayerName;
	}

	public class clsQueryHelper
	{

		public static List<ArcGISDynamicMapServiceLayer> DynamicLayers = null;
		public static Dictionary<int, Dictionary<int, FeatureLayerInfo>> Sublayers = null;
		public static List<QueryLayer> QueryableLayers = null;

		private static bool _bFlash = false;
		private static bool _bInitialized = false;
		private static Map _Map = null;
		private static clsCommonHelper _ch;
		private static clsGeometryHelper _gh;
		private static clsGraphicHelper _grh;
		private static Geometry _geom;

		#region Public Methods and Events
		public void Init(Map MyMap)
		{

			if (_Map != null)
				return;
			_Map = MyMap;
			_Map.Progress += new EventHandler<ProgressEventArgs>(_Map_Progress);
			_ch = clsHelperCollection.CommonHelper;
			string sUrl = clsHelperCollection.GeometryHelper.Url;
			_gh = new clsGeometryHelper();
			_gh.Initialize(sUrl);
			_gh.GetCenterCompleted += new EventHandler<GeometryResultArgs>(_gh_GetCenterCompleted);
			_gh.Failed += new EventHandler<HelperFailedArgs>(_gh_Failed);
			_grh = clsHelperCollection.GraphicHelper;
			this.UpdateLayers();

		}

		public void UpdateLayers()
		{
	
			// Get reverse order list of dynamic layers

			DynamicLayers = new List<ArcGISDynamicMapServiceLayer>();
			foreach (Layer l in _Map.Layers)
			{
				if (!(l is ArcGISDynamicMapServiceLayer))
					continue;
				DynamicLayers.Add((ArcGISDynamicMapServiceLayer)l);
			}
			DynamicLayers.Reverse();

			// Get sublayer details

			Sublayers = new Dictionary<int, Dictionary<int, FeatureLayerInfo>>();
			foreach (ArcGISDynamicMapServiceLayer dynLayer in DynamicLayers)
				dynLayer.GetAllDetails(this.AddLayer);

		}

		public event EventHandler<EventArgs> QueryHelperInitialized;
		public event EventHandler<EventArgs> QueryHelperUpdated;

		public static void Flash(Geometry g, bool bCrossHair = true)
		{
			if (_Map == null)
				return;
			if (g is MultiPoint)
			{
				MultiPoint mp = (MultiPoint)g;
				foreach (MapPoint pt in mp.Points)
				{
					MapPoint ptc = null;
					if (bCrossHair)
						ptc = pt;
					_grh.FlashGeometry(_Map, pt, ptc);
				}
			}
			if (!bCrossHair)
			{
				_grh.FlashGeometry(_Map, g, null);
				return;
			}
			_gh.GetCenterAsync(g, g);
		}

		public static void Zoom(Geometry g, bool bFlash)
		{
			if (_Map == null)
				return;
			_bFlash = bFlash;
			_geom = g;
			_ch.ZoomTo(_Map, g);
		}

		public static void Pan(Geometry g, bool bFlash)
		{
			if (_Map == null)
				return;
			_bFlash = bFlash;
			_geom = g;
			_Map.PanTo(g);
		}
		#endregion

		#region Event Handlers
		private static void _Map_Progress(object sender, ProgressEventArgs e)
		{
			if (!_bFlash)
				return;
			if (e.Progress < 100)
				return;
			_bFlash = false;
			Flash(_geom);
		}

		private static void _gh_GetCenterCompleted(object sender, GeometryResultArgs e)
		{
			Geometry g = (Geometry)e.UserState;
			MapPoint ptCenter = (MapPoint)e.Geometry;
			_grh.FlashGeometry(_Map, g, ptCenter);
		}

		private static void _gh_Failed(object sender, HelperFailedArgs e)
		{
			Error("Geometry operation failed:\n" + e.Error.Message);
		}
		#endregion

		#region Private Methods
		private static void Error(string sErr)
		{
			MessageBox.Show(sErr, "Query Helper", MessageBoxButton.OK);
		}

		private void AddLayer(IDictionary<int, FeatureLayerInfo> details, Exception ex)
		{

			if (ex != null)
			{
				string sMsg = "Error getting service details:\n" + ex.Message;
				MessageBox.Show(sMsg, "Query Helper", MessageBoxButton.OK);
				return;
			}

			// Get matching dynamic layer

			bool bFound = false;
			int iLayer = 0;
			string sFirstLayer = details[0].Name;
			foreach (ArcGISDynamicMapServiceLayer dl in DynamicLayers)
			{
				if (dl.Layers[0].Name == sFirstLayer)
				{
					bFound = true;
					break;
				}
				iLayer++;
			}
			if (!bFound)
				return; // This should never happen

			// Filter out group layers, subclasses, etc.

			Dictionary<int, FeatureLayerInfo> SublayerDetails = new Dictionary<int, FeatureLayerInfo>();
			foreach (int iKey in details.Keys)
			{
				FeatureLayerInfo info = details[iKey];
				bool bSkip = true;
				switch (info.Type)
				{
					case "Feature Layer":
					case "Annotation Layer":
					case "Table":
						bSkip = false;
						break;
				}
				if (bSkip)
					continue;
				SublayerDetails[iKey] = info;
			}
			Sublayers[iLayer] = SublayerDetails;
			if (Sublayers.Count < DynamicLayers.Count)
				return;  // Not yet ready to populate queryable layer list

			// Build queryable layer list

			QueryableLayers = new List<QueryLayer>();
			iLayer = 0;
			foreach (ArcGISDynamicMapServiceLayer dynLayer in DynamicLayers)
			{
				SublayerDetails = Sublayers[iLayer];
				foreach (LayerInfo li in dynLayer.Layers)
				{
					int id = li.ID;
					string sName = li.Name;
					if (!SublayerDetails.ContainsKey(id))
						continue;
					FeatureLayerInfo Info = SublayerDetails[id];
					//if (Info.Type == "Table")
					//   continue; // This should never happen
					QueryLayer ql = new QueryLayer();
					ql.LayerIndex = iLayer;
					ql.SublayerID = id;
					ql.SublayerName = sName;
					QueryableLayers.Add(ql);
				}
				iLayer++;
			}

			if (!_bInitialized)
			{
				_bInitialized = true;
				this.QueryHelperInitialized(this, new EventArgs());
			}
			else
				this.QueryHelperUpdated(this, new EventArgs());

		}
		#endregion

	}
}
