﻿using System;
using System.Reflection;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;

using FieldApp.Extensibility;

namespace NavAddin
{

	// Convenient base class for navigation commands
	#region NavButton
	public abstract class NavButton : IMapButton
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
		protected string _sButtonContent;
		protected string _sIconPath;
		protected string _sToolTip;

		#region Overridable Methods
		abstract protected bool NavEnabled();

		abstract protected void NavInitialized();

		abstract protected void NavExtentUpdated();

		abstract protected void NavExecute();
		#endregion

		#region IMapButton Implementation
		public object ButtonContent
		{
			get 
			{
				if (_sButtonContent != null)
					return _sButtonContent;
				else if (_sIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sIconPath);
				return null;
			}
		}

		public bool IsEnabled
		{
			get 
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					NavBehavior nb = NavBehavior.Current;
					nb.MapExtentUpdated += new EventHandler(nb_MapExtentUpdated);
					if (!nb.IsInitialized)
					{
						nb.Initialized += new EventHandler(nb_Initialized);
						return false;
					}
					_bMapEnabled = true;
				}
				if (!_bMapEnabled)
					return false;
				return this.NavEnabled();
			}
		}

		public string ToolTip
		{
			get { return _sToolTip; }
		}

		public void OnClick()
		{
			this.NavExecute();
		}

		public event EventHandler<EventArgs> IsEnabledChanged;
		#endregion

		protected virtual void OnIsEnabledChanged()
		{
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

		void nb_Initialized(object sender, EventArgs e)
		{
			_bMapEnabled = true;
			this.NavInitialized();
			this.OnIsEnabledChanged();
		}

		void nb_MapExtentUpdated(object sender, EventArgs e)
		{
			this.NavExtentUpdated();
		}

	}
	#endregion

	// Convenient base class for navigation tools
	#region NavTool
	public abstract class NavTool : IMapTool
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
		protected string _sButtonContent;
		protected string _sIconPath;
		protected string _sToolTip;
		protected string _sCursorPath;

		#region Overridable Methods
		abstract protected void NavInitialized();

		abstract protected bool NavEnabled();

		abstract protected void NavExtentUpdated();

		abstract protected void NavActivateTool();

		abstract protected void NavDeactivateTool();
		#endregion

		#region IMapTool Implementation
		public object ButtonContent
		{
			get 
			{
				if (_sButtonContent != null)
					return _sButtonContent;
				else if (_sIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sIconPath);
				return null;
			}
		}

		public Cursor MouseCursor
		{
			get 
			{
				if (_sCursorPath == null)
					return Cursors.Arrow;
				return AddinHelper.GetCursor(_MyAssembly, _sCursorPath);
			}
		}

		public bool IsEnabled
		{
			get 
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					NavBehavior nb = NavBehavior.Current;
					nb.MapExtentUpdated += new EventHandler(nb_MapExtentUpdated);
					if (!nb.IsInitialized)
					{
						nb.Initialized += new EventHandler(nb_Initialized);
						return false;
					}
					_bMapEnabled = true;
				}
				if (!_bMapEnabled)
					return false;
				return this.NavEnabled();
			}
		}

		public string ToolTip
		{
			get { return _sToolTip; }
		}

		public void Activate()
		{
			this.NavActivateTool();
		}

		public void Deactivate()
		{
			this.NavDeactivateTool();
		}

		public event EventHandler<EventArgs> IsEnabledChanged;
		#endregion

		protected virtual void OnIsEnabledChanged()
		{
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

		void nb_Initialized(object sender, EventArgs e)
		{
			_bMapEnabled = true;
			this.NavInitialized();
			this.OnIsEnabledChanged();
		}

		void nb_MapExtentUpdated(object sender, EventArgs e)
		{
			this.NavExtentUpdated();
		}

	}
	#endregion

	#region NavMenuItem
	public abstract class NavMenuItem : IMapMenuItem
	{

		private bool _bInitialized = false;
		private bool _bMapEnabled = false;

		protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
		protected string _sLabel;
		protected string _sIconPath;
		protected bool _bIsCheckable;
		protected bool _bIsChecked;

		#region Overridable Methods
		abstract protected void NavExecute();
		#endregion

		#region IMapMenuItem implementation
		public Image Icon
		{
			get
			{
				if (_sIconPath != null)
					return AddinHelper.GetImage(_MyAssembly, _sIconPath);
				return null;
			}
		}

		public bool IsCheckable
		{
			get { return _bIsCheckable; }
		}

		public bool IsChecked
		{
			get { return _bIsChecked; }
		}

		public bool IsEnabled
		{
			get
			{
				if (!_bInitialized)
				{
					_bInitialized = true;
					NavBehavior nb = NavBehavior.Current;
					if (!nb.IsInitialized)
					{
						nb.Initialized += new EventHandler(nb_Initialized);
						return false;
					}
					_bMapEnabled = true;
				}
				return _bMapEnabled;
			}
		}

		public event EventHandler<EventArgs> IsEnabledChanged;

		public string Label
		{
			get { return _sLabel; }
		}

		public void OnClick()
		{
			this.NavExecute();
		}
		#endregion

		void nb_Initialized(object sender, EventArgs e)
		{
			_bMapEnabled = true;
			if (IsEnabledChanged != null)
				IsEnabledChanged(this, new EventArgs());
		}

	}
	#endregion

}
