﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.FeatureService;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

using MapHelpers;
using FieldApp.Extensibility;

namespace QueryAddin
{
	/// <summary>
	/// Interaction logic for ucFind.xaml
	/// </summary>
	public partial class ucFind : UserControl, IExpandableContent
	{

		private class FieldNameEntry
		{
			public string Alias = "";
			public HashSet<string> Names = new HashSet<string>();
			public override string ToString()
			{
				return this.Alias;
			}
		}

		private const double BaseHeight = 230;
		private const double ExpanderHeight = 200;
		private const int MaxResults = 500;

		private bool _bInitialized = false;
		private bool _bExpander = false;
		private Map _Map;
		private clsCommonHelper _ch;
		private ucIdentify _ucIdentify;
		private int _iLayerIndex = 0;
		private List<FindTask> _FindTasks;
		private List<LayerSearchItem> _QueryableLayers;
		private LayerSearchType _SearchType;
		private int _iResultCount = 0;
		private ObservableCollection<SearchFeature> _ResultList;

		#region Public Methods
		public ucFind()
		{
			InitializeComponent();
			Height = BaseHeight;
			grdResults.LayoutUpdated += grdResults_LayoutUpdated;
			this.SizeChanged += ucFind_SizeChanged;
		}

		public void Initialize(Map MyMap, ucIdentify MyIdentify)
		{

			if (_bInitialized)
				return;

			_Map = MyMap;
			_ch = clsHelperCollection.CommonHelper;
			_ucIdentify = MyIdentify;

			// Control events

			cbLayers.SelectionChanged += new SelectionChangedEventHandler(cbLayers_SelectionChanged);
			rbAllFields.Checked += new RoutedEventHandler(rbAllFields_Checked);
			rbInField.Checked += new RoutedEventHandler(rbInField_Checked);
			btnFind.Click += new RoutedEventHandler(btnFind_Click);
			btnNew.Click += btnNew_Click;

			QueryBehavior qb = QueryBehavior.Current;
			qb.LayersUpdated += new EventHandler<EventArgs>(qb_LayersUpdated);
			this.RefreshLayers();
			_bInitialized = true;

		}
		#endregion

		#region IExpandableContent implementation
		// This is not needed in Silverlight but is kept for WPF compatibility
		public event EventHandler<EventArgs> ContentSizeChanged;
		#endregion

		#region XAML Declared Event Handlers
		private void miFlash_Click(object sender, RoutedEventArgs e)
		{
			MenuItem mi = (MenuItem)sender;
			SearchFeature searchFeat = (SearchFeature)mi.DataContext;
			Geometry g = searchFeat.Feature.Geometry;
			clsQueryHelper.Flash(g, true);
		}

		private void miZoomTo_Click(object sender, RoutedEventArgs e)
		{
			MenuItem mi = (MenuItem)sender;
			SearchFeature searchFeat = (SearchFeature)mi.DataContext;
			Geometry g = searchFeat.Feature.Geometry;
			clsQueryHelper.Zoom(g, true);
		}

		private void miPanTo_Click(object sender, RoutedEventArgs e)
		{
			MenuItem mi = (MenuItem)sender;
			SearchFeature searchFeat = (SearchFeature)mi.DataContext;
			Geometry g = searchFeat.Feature.Geometry;
			clsQueryHelper.Pan(g, true);
		}

		private void miIdentify_Click(object sender, RoutedEventArgs e)
		{
			MenuItem mi = (MenuItem)sender;
			SearchFeature searchFeat = (SearchFeature)mi.DataContext;
			_ucIdentify.IdentifyFeature(searchFeat);
		}
		#endregion

		#region Event Handlers
		void grdResults_LayoutUpdated(object sender, EventArgs e)
		{
			if (!_bExpander)
				return;
			_bExpander = false;
			if (grdResults.Visibility == System.Windows.Visibility.Visible)
			{
				this.Height = BaseHeight + ExpanderHeight;
				this.MaxHeight = double.PositiveInfinity;
			}
			else
			{
				this.MaxHeight = BaseHeight;
				this.Height = BaseHeight;
			}
			if (this.ContentSizeChanged != null)
				this.ContentSizeChanged(this, new EventArgs());
		}

		void ucFind_SizeChanged(object sender, SizeChangedEventArgs e)
		{
			DependencyObject doParent = this.Parent;
			if (!(doParent is FrameworkElement))
				return;
			FrameworkElement feParent = (FrameworkElement)doParent;
			double dHeight = feParent.ActualHeight;
			if (double.IsNaN(dHeight))
				return;
			if (dHeight > BaseHeight)
				this.Height = dHeight;
		}

		private void cbLayers_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			LayerSearchItem lsItem = (LayerSearchItem)cbLayers.SelectedItem;
			if (lsItem == null)
			{
				cbFields.Items.Clear();
				return;
			}
			_SearchType = lsItem.SearchType;
			if (rbInField.IsChecked == true)
				this.PopulateFieldOptions();
		}

		private void rbAllFields_Checked(object sender, RoutedEventArgs e)
		{
			cbFields.IsEnabled = false;
		}

		private void rbInField_Checked(object sender, RoutedEventArgs e)
		{
			this.PopulateFieldOptions();
			cbFields.IsEnabled = true;
		}

		private void btnFind_Click(object sender, RoutedEventArgs e)
		{

			_iResultCount = 0;
			dgResults.ItemsSource = null;
			_bExpander = true;
			grdResults.Visibility = System.Windows.Visibility.Visible;
			_ResultList = new ObservableCollection<SearchFeature>();
			this.Status("Querying...");
			_iLayerIndex = 0;
			FindParameters findParams = this.GetFindParameters();
			FindTask findTask = _FindTasks[_iLayerIndex];
			findTask.ExecuteAsync(findParams);

		}

		void btnNew_Click(object sender, RoutedEventArgs e)
		{
			_bExpander = true;
			dgResults.ItemsSource = null;
			grdResults.Visibility = System.Windows.Visibility.Collapsed;
			txtFind.Text = "";
			chkLike.IsChecked = true;
			rbAllFields.IsChecked = true;
		}

		private void FindTask_ExecuteCompleted(object sender, FindEventArgs e)
		{

			// Check for no results

			bool bNoResults = false;
			if (e.FindResults == null)
				bNoResults = true;
			else if (e.FindResults.Count == 0)
				bNoResults = true;
			if (bNoResults)
			{

				// See if next dynamic layer needs to be queried

				if (_SearchType != LayerSearchType.Single)
				{
					if (_iLayerIndex < clsQueryHelper.DynamicLayers.Count - 1)
					{
						_iLayerIndex += 1;
						FindParameters findParams = GetFindParameters();
						FindTask findTask = _FindTasks[_iLayerIndex];
						findTask.ExecuteAsync(findParams);
						return;
					}
				}

				// Nothing found

				MessageBox.Show("No features found.", "Find", MessageBoxButton.OK);
				this.Status("OK");
				return;

			}

			// Populate datagrid

			this.Status("Building results...");
			foreach (FindResult findResult in e.FindResults)
			{
				if (_iResultCount == MaxResults)
					break;
				SearchFeature sf = new SearchFeature();
				Graphic gFeat = findResult.Feature;
				sf.Feature = gFeat;
				if (gFeat.Attributes.ContainsKey("OBJECTID"))
					sf.ObjectID = Convert.ToInt32(gFeat.Attributes["OBJECTID"]);
				sf.Value = findResult.Value.ToString();
				sf.LayerName = findResult.LayerName;
				sf.FieldName = findResult.FoundFieldName;
				sf.LayerIndex = _iLayerIndex;
				_ResultList.Add(sf);
				_iResultCount++;
			}

			// See if next dynamic layer needs to be queried

			if ((_iResultCount < MaxResults) && (_SearchType != LayerSearchType.Single))
			{
				if (_iLayerIndex < clsQueryHelper.DynamicLayers.Count - 1)
				{
					_iLayerIndex += 1;
					FindParameters findParams = GetFindParameters();
					FindTask findTask = _FindTasks[_iLayerIndex];
					findTask.ExecuteAsync(findParams);
					return;
				}
			}

			// Done

			string sMsg;
			if (_iResultCount == 1)
				sMsg = "One object found";
			else
				sMsg = _iResultCount.ToString() + " objects found";
			this.Status(sMsg);
			dgResults.ItemsSource = _ResultList;
			dgResults.ScrollIntoView(_ResultList[0], dgResults.Columns[0]);

		}

		private void FindTask_Failed(object sender, TaskFailedEventArgs e)
		{
			this.Error("Identify failed:\n" + e.Error.Message);
		}

		private void qb_LayersUpdated(object sender, EventArgs e)
		{
			this.RefreshLayers();
		}
		#endregion

		#region Private Methods
		private void Status(string sMsg)
		{
			lblResults.Content = sMsg;
			//Helper.DoEvents();
		}
		private void Error(string sErr)
		{
			lblResults.Content = "ERROR";
			MessageBox.Show(sErr, "ERROR", MessageBoxButton.OK);
		}

		private void RefreshLayers()
		{

			// Set up find tasks

			_FindTasks = new List<FindTask>();
			foreach (ArcGISDynamicMapServiceLayer dynLayer in clsQueryHelper.DynamicLayers)
			{
				string sURL = dynLayer.Url;
				FindTask FTask = new FindTask(sURL);
				FTask.ExecuteCompleted += new EventHandler<FindEventArgs>(FindTask_ExecuteCompleted);
				FTask.Failed += new EventHandler<TaskFailedEventArgs>(FindTask_Failed);
				_FindTasks.Add(FTask);
			}

			// Populate layer options

			cbLayers.Items.Clear();
			LayerSearchType[] Types = { LayerSearchType.Visible, LayerSearchType.All };
			foreach (LayerSearchType opt in Types)
			{
				LayerSearchItem lsItem = new LayerSearchItem();
				lsItem.SearchType = opt;
				cbLayers.Items.Add(lsItem);
			}
			_QueryableLayers = new List<LayerSearchItem>();
			foreach (QueryLayer ql in clsQueryHelper.QueryableLayers)
			{
				LayerSearchItem lsItem = new LayerSearchItem();
				lsItem.SearchType = LayerSearchType.Single;
				lsItem.LayerIndex = ql.LayerIndex;
				lsItem.SubLayerID = ql.SublayerID;
				lsItem.SubLayerName = ql.SublayerName;
				cbLayers.Items.Add(lsItem);
				_QueryableLayers.Add(lsItem);
			}
			cbLayers.SelectedIndex = 0;
			_SearchType = LayerSearchType.Visible;

		}

		private void PopulateFieldOptions()
		{

			// Get layers

			bool bSingle = (_SearchType == LayerSearchType.Single);
			List<LayerSearchItem> LayerList = new List<LayerSearchItem>();
			if (bSingle)
			{
				LayerSearchItem lsItem = (LayerSearchItem)cbLayers.SelectedItem;
				LayerList.Add(lsItem);
			}
			else
			{
				foreach (LayerSearchItem lsItem in _QueryableLayers)
				{
					if (_SearchType == LayerSearchType.Visible)
					{
						if (!this.IsLayerVisible(lsItem))
							continue;
					}
					LayerList.Add(lsItem);
				}
			}

			// Build field name entries

			Dictionary<string, FieldNameEntry> FieldEntryList = new Dictionary<string, FieldNameEntry>();
			List<string> AliasList = new List<string>();
			foreach (LayerSearchItem lsItem in LayerList)
			{
				FeatureLayerInfo Info = _ucIdentify.GetSublayerInfo(lsItem.LayerIndex, lsItem.SubLayerID);
				if (Info == null)
					continue; // This shouldn't happen if _ucIdentify is initialized
				foreach (Field FName in Info.Fields)
				{
					string sAlias = FName.Alias;
					FieldNameEntry FNEntry;
					if (FieldEntryList.ContainsKey(sAlias))
						FNEntry = FieldEntryList[sAlias];
					else
					{
						if (bSingle)
							AliasList.Add(sAlias);
						FNEntry = new FieldNameEntry();
						FNEntry.Alias = sAlias;
						FieldEntryList[sAlias] = FNEntry;
					}
					FNEntry.Names.Add(FName.Name);
				}
			}

			// Populate fields combobox

			cbFields.Items.Clear();
			if (!bSingle)
			{
				AliasList = FieldEntryList.Keys.ToList();
				AliasList.Sort();
			}
			foreach (string sName in AliasList)
				cbFields.Items.Add(FieldEntryList[sName]);
			cbFields.SelectedIndex = 0;

		}

		private bool IsLayerVisible(LayerSearchItem lsItem)
		{
			ArcGISDynamicMapServiceLayer dynLayer = clsQueryHelper.DynamicLayers[_iLayerIndex];
			int iLayerID = lsItem.SubLayerID;
			return _ch.IsSublayerVisible(dynLayer, iLayerID, 0);
		}

		private FindParameters GetFindParameters()
		{

			FindParameters findParams = new FindParameters();
			findParams.SpatialReference = _Map.SpatialReference;
			findParams.SearchText = txtFind.Text;
			List<int> LayerIds = findParams.LayerIds;
			LayerIds.Clear();
			if (_SearchType == LayerSearchType.Single)
			{
				LayerSearchItem lsItem = (LayerSearchItem)cbLayers.SelectedItem;
				LayerIds.Add(lsItem.SubLayerID);
				_iLayerIndex = lsItem.LayerIndex;
			}
			else
			{
				foreach (LayerSearchItem lsItem in _QueryableLayers)
				{
					if (lsItem.LayerIndex != _iLayerIndex)
						continue;
					if (_SearchType == LayerSearchType.Visible)
					{
						if (!this.IsLayerVisible(lsItem))
							continue;
					}
					LayerIds.Add(lsItem.SubLayerID);
				}
			}
			findParams.Contains = (chkLike.IsChecked == true);
			List<string> SearchFields = findParams.SearchFields;
			SearchFields.Clear();
			if (rbAllFields.IsChecked == true)
				return findParams;
			FieldNameEntry fnEntry = (FieldNameEntry)cbFields.SelectedItem;
			foreach (string sName in fnEntry.Names)
				SearchFields.Add(sName);
			return findParams;

		}
		#endregion

	}
}
