﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Tasks;

using FieldApp.Extensibility;
using MapHelpers;

namespace QueryAddin
{
	/// <summary>
	/// Interaction logic for ucMeasure.xaml
	/// </summary>
	public partial class ucMeasure : UserControl
	{

		private enum MeasureMode
		{
			Line,
			Poly,
			Feature
		}

		private bool _bInitialized = false;
		private Map _Map;
		private IParentWindow _pParent;
		private ucIdentify _ucIdentify;
		private esriUnits _MapUnits;
		private esriUnits _DisplayUnits;
		private AreaUnit _MapAreaUnits;
		private AreaUnit _DisplayAreaUnits;
		private clsGeometryHelper _gh;
		private clsGraphicHelper _grh;
		private MeasureMode _Mode;
		private Draw _MeasureDrawObject;
		private bool _bRubberBand = false;
		private List<Geometry> _GeomList;
		private bool _bShowSum = false;

		#region Public Methods
		public ucMeasure()
		{
			InitializeComponent();
			this.ClearResults();
		}

		public void Initialize(Map MyMap, IParentWindow pParent, ucIdentify Identify)
		{

			if (_bInitialized)
				return;

			// Assume map units will be meters, feet, or degrees

			_Map = MyMap;
			_Map.MouseMove += new MouseEventHandler(_Map_MouseMove);
			_pParent = pParent;
			_ucIdentify = Identify;
			_MapUnits = clsHelperCollection.MapUnits;
			_DisplayUnits = _MapUnits;
			_MapAreaUnits = AreaUnit.Default;
			if (_DisplayUnits == esriUnits.esriMeters)
			{
				_MapAreaUnits = AreaUnit.SquareMeters;
				miMeters.IsChecked = true;
				miSqMeters.IsChecked = true;
			}
			else if (_DisplayUnits == esriUnits.esriFeet)
			{
				_MapAreaUnits = AreaUnit.SquareFeet;
				miFeet.IsChecked = true;
				miSqFeet.IsChecked = true;
			}
			_DisplayAreaUnits = _MapAreaUnits;
			_gh = clsHelperCollection.GeometryHelper;
			_grh = clsHelperCollection.GraphicHelper;
			_Mode = MeasureMode.Line;
			_MeasureDrawObject = new Draw(_Map);
			_MeasureDrawObject.DrawMode = DrawMode.Polyline;
			_MeasureDrawObject.IsEnabled = false;
			_MeasureDrawObject.DrawBegin += new EventHandler(_MeasureDrawObject_DrawBegin);
			_MeasureDrawObject.VertexAdded += new EventHandler<VertexAddedEventArgs>(_MeasureDrawObject_VertexAdded);
			_MeasureDrawObject.DrawComplete += new EventHandler<DrawEventArgs>(_MeasureDrawObject_DrawComplete);
			_GeomList = new List<Geometry>();

			// Set up control handlers

			rbLine.Checked += new RoutedEventHandler(rbLine_Checked);
			rbPoly.Checked += new RoutedEventHandler(rbPoly_Checked);
			rbFeature.Checked += new RoutedEventHandler(rbFeature_Checked);
			chkSum.Checked += new RoutedEventHandler(chkSum_Checked);
			chkSum.Unchecked += new RoutedEventHandler(chkSum_Unchecked);
			btnUnits.Click += new RoutedEventHandler(btnUnits_Click);
			miKilometers.Click += new RoutedEventHandler(miKilometers_Click);
			miMeters.Click += new RoutedEventHandler(miMeters_Click);
			miDecimeters.Click += new RoutedEventHandler(miDecimeters_Click);
			miCentimeters.Click += new RoutedEventHandler(miCentimeters_Click);
			miMillimeters.Click += new RoutedEventHandler(miMillimeters_Click);
			miMiles.Click += new RoutedEventHandler(miMiles_Click);
			miNauticalMiles.Click += new RoutedEventHandler(miNauticalMiles_Click);
			miYards.Click += new RoutedEventHandler(miYards_Click);
			miFeet.Click += new RoutedEventHandler(miFeet_Click);
			miInches.Click += new RoutedEventHandler(miInches_Click);
			miSqKilometers.Click += new RoutedEventHandler(miSqKilometers_Click);
			miHectares.Click += new RoutedEventHandler(miHectares_Click);
			miSqMeters.Click += new RoutedEventHandler(miSqMeters_Click);
			miAres.Click += new RoutedEventHandler(miAres_Click);
			miSqMiles.Click += new RoutedEventHandler(miSqMiles_Click);
			miAcres.Click += new RoutedEventHandler(miAcres_Click);
			miSqFeet.Click += new RoutedEventHandler(miSqFeet_Click);
			miSqYards.Click += new RoutedEventHandler(miSqYards_Click);
			btnClear.Click += new RoutedEventHandler(btnClear_Click);
	
			rbLine.IsChecked = true;
			_bInitialized = true;

		}

		public void ActivateTool()
		{
			if (_pParent != null)
				_pParent.ShowWindow();
			if (_MeasureDrawObject != null)
				_MeasureDrawObject.IsEnabled = true;
		}

		public void DeactivateTool()
		{
			if (_MeasureDrawObject != null)
				_MeasureDrawObject.IsEnabled = false;
		}

		public void IdentifyGeometryCallback(Geometry g)
		{
			if (g == null)
			{
				MessageBox.Show("No feature found.");
				return;
			}
			_grh.FlashGeometry(_Map, g, null);
			if (g is MapPoint)
			{
				this.UpdatePoint((MapPoint)g);
				return;
			}
			_GeomList.Insert(0, g);
			this.UpdateMeasure();
		}
		#endregion

		#region Control Handlers
		private void rbLine_Checked(object sender, RoutedEventArgs e)
		{
			this.ClearResults();
			_MeasureDrawObject.DrawMode = DrawMode.Polyline;
			_Mode = MeasureMode.Line;
		}

		private void rbPoly_Checked(object sender, RoutedEventArgs e)
		{
			this.ClearResults();
			_MeasureDrawObject.DrawMode = DrawMode.Polygon;
			_Mode = MeasureMode.Poly;
		}

		private void rbFeature_Checked(object sender, RoutedEventArgs e)
		{
			this.ClearResults();
			_MeasureDrawObject.DrawMode = DrawMode.Point;
			_Mode = MeasureMode.Feature;
		}

		private void chkSum_Checked(object sender, RoutedEventArgs e)
		{
			_bShowSum = true;
			this.UpdateMeasure();
		}

		private void chkSum_Unchecked(object sender, RoutedEventArgs e)
		{
			_bShowSum = false;
			this.UpdateMeasure();
		}

		private void btnUnits_Click(object sender, RoutedEventArgs e)
		{
			btnUnits.ContextMenu.IsOpen = true;
		}

		private void miKilometers_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miKilometers.IsChecked = true;
			_DisplayUnits = esriUnits.esriKilometers;
			this.UpdateMeasure();
		}

		private void miMeters_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miMeters.IsChecked = true;
			_DisplayUnits = esriUnits.esriMeters;
			this.UpdateMeasure();
		}

		private void miDecimeters_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miDecimeters.IsChecked = true;
			_DisplayUnits = esriUnits.esriDecimeters;
			this.UpdateMeasure();
		}

		private void miCentimeters_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miCentimeters.IsChecked = true;
			_DisplayUnits = esriUnits.esriCentimeters;
			this.UpdateMeasure();
		}

		private void miMillimeters_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miMillimeters.IsChecked = true;
			_DisplayUnits = esriUnits.esriMillimeters;
			this.UpdateMeasure();
		}

		private void miMiles_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miMiles.IsChecked = true;
			_DisplayUnits = esriUnits.esriMiles;
			this.UpdateMeasure();
		}

		private void miNauticalMiles_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miNauticalMiles.IsChecked = true;
			_DisplayUnits = esriUnits.esriNauticalMiles;
			this.UpdateMeasure();
		}

		private void miYards_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miYards.IsChecked = true;
			_DisplayUnits = esriUnits.esriYards;
			this.UpdateMeasure();
		}

		private void miFeet_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miFeet.IsChecked = true;
			_DisplayUnits = esriUnits.esriFeet;
			this.UpdateMeasure();
		}

		private void miInches_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckLengths();
			miInches.IsChecked = true;
			_DisplayUnits = esriUnits.esriInches;
			this.UpdateMeasure();
		}

		private void miSqKilometers_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miSqKilometers.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.SquareKilometers;
			this.UpdateMeasure();
		}

		private void miHectares_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miHectares.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.Hectares;
			this.UpdateMeasure();
		}

		private void miSqMeters_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miSqMeters.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.SquareMeters;
			this.UpdateMeasure();
		}

		private void miAres_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miAres.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.Ares;
			this.UpdateMeasure();
		}

		private void miSqMiles_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miSqMiles.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.SquareMiles;
			this.UpdateMeasure();
		}

		private void miAcres_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miAcres.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.Acres;
			this.UpdateMeasure();
		}

		private void miSqFeet_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miSqFeet.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.SquareFeet;
			this.UpdateMeasure();
		}

		private void miSqYards_Click(object sender, RoutedEventArgs e)
		{
			this.UncheckAreas();
			miSqYards.IsChecked = true;
			_DisplayAreaUnits = AreaUnit.SquareYards;
			this.UpdateMeasure();
		}

		private void btnClear_Click(object sender, RoutedEventArgs e)
		{
			this.ClearResults();
		}
		#endregion

		#region Draw handlers
		private void _MeasureDrawObject_DrawBegin(object sender, EventArgs e)
		{
			switch (_Mode)
			{
				case MeasureMode.Line:
					_GeomList.Insert(0, new Polyline());
					_bRubberBand = true;
					break;
				case MeasureMode.Poly:
					_GeomList.Insert(0, new Polygon());
					_bRubberBand = true;
					break;
				case MeasureMode.Feature:
					_bRubberBand = false;
					break;
			}
		}

		private void _Map_MouseMove(object sender, MouseEventArgs e)
		{
			if (!_bRubberBand)
				return;
			MapPoint pt = _Map.ScreenToMap(e.GetPosition(_Map));
			this.Measure(pt, false);
		}

		void _MeasureDrawObject_VertexAdded(object sender, VertexAddedEventArgs e)
		{
			MapPoint pt = e.Vertex;
			this.Measure(pt, true);
		}

		private void _MeasureDrawObject_DrawComplete(object sender, DrawEventArgs e)
		{
			_bRubberBand = false;
			if (_Mode != MeasureMode.Feature)
			{
				_GeomList.RemoveAt(0);
				_GeomList.Insert(0, e.Geometry);
				this.UpdateMeasure();
				return;
			}
			MapPoint pt = (MapPoint)e.Geometry;
			
			// Identify topmost feature

			_ucIdentify.IdentifyGeometry(pt, this.IdentifyGeometryCallback);
		
		}
		#endregion

		#region Private Methods

		private void ClearResults()
		{
			if (_GeomList != null)
				_GeomList.Clear();
			string sInit = "To measure a distance, draw a line.\n\n";
			sInit += "To measure an area, click 'Measure An Area'\nthen draw a polygon.\n\n";
			sInit += "To measure a feature, click 'Measure A Feature'\nthen click a feature.";
			txtResults.Text = sInit;
		}

		private void UncheckLengths()
		{
			miKilometers.IsChecked = false;
			miMeters.IsChecked = false;
			miDecimeters.IsChecked = false;
			miCentimeters.IsChecked = false;
			miMillimeters.IsChecked = false;
			miMiles.IsChecked = false;
			miNauticalMiles.IsChecked = false;
			miYards.IsChecked = false;
			miFeet.IsChecked = false;
			miInches.IsChecked = false;
		}

		private void UncheckAreas()
		{
			miSqKilometers.IsChecked = false;
			miHectares.IsChecked = false;
			miSqMeters.IsChecked = false;
			miAres.IsChecked = false;
			miSqMiles.IsChecked = false;
			miAcres.IsCheckable = false;
			miSqFeet.IsChecked = false;
			miSqYards.IsChecked = false;
		}

		private void Measure(MapPoint pt, bool bAddPoint)
		{

			bool bTest = false;
			if (bTest)
			{
				string sTxt = "X: " + pt.X.ToString() + "\nY: " + pt.Y.ToString();
				txtResults.Text = sTxt;
				return;
			}

			// Add point to geometry

			Geometry g = _GeomList[0];
			bool bLine = (g is Polyline);
			PointCollection PC;
			if (bLine)
			{
				Polyline p = (Polyline)g;
				if (p.Paths.Count == 0)
				{
					PC = new PointCollection();
					PC.Add(pt);
					p.Paths.Add(PC);
				}
				else
				{
					PC = p.Paths[p.Paths.Count - 1];
					PC.Add(pt);
				}
			}
			else
			{
				Polygon p = (Polygon)g;
				if (p.Rings.Count == 0)
				{
					PC = new PointCollection();
					PC.Add(pt);
					PC.Add(pt);
					p.Rings.Add(PC);
				}
				else 
				{
					PC = p.Rings[p.Rings.Count - 1];
					PC.Insert(PC.Count - 1, pt);
				}
			}

			UpdateMeasure();
			if (bAddPoint)
				return;

			// Remove point from geometry

			if (bLine)
			{
				Polyline p = (Polyline)g;
				PC = p.Paths[p.Paths.Count - 1];
				PC.RemoveAt(PC.Count - 1);
			}
			else
			{
				Polygon p = (Polygon)g;
				PC = p.Rings[p.Rings.Count - 1];
				if (PC.Count == 2)
					PC.Clear();
				else
					PC.RemoveAt(PC.Count - 2);
			}

		}

		private void UpdateMeasure()
		{

			// Get length and area values

			if (_GeomList.Count == 0)
				return;
			double dLen = 0;
			double dSegLen = 0;
			double dArea = 0;
			double dSumLen = 0;
			double dSumArea = 0;
			bool bFirst = true;
			Geometry gFirst = null;
			bool bLine = true;
			
			foreach (Geometry g in _GeomList)
			{

				if (bFirst)
				{
					gFirst = g;
					bLine = (gFirst is Polyline);
				}
				if (bLine && (g is Polygon))
					continue;
				else if ((!bLine) && (g is Polyline))
					continue;

				ObservableCollection<PointCollection> PCList;
				double dL = 0;
				double dA = 0;
				if (bLine)
				{
					Polyline p = (Polyline)g;
					PCList = p.Paths;
					
					dL = Euclidian.Length(p);
				}
				else
				{
					Polygon p = (Polygon)g;
					PCList = p.Rings;
					dL = Euclidian.Length(p);
					dA = Euclidian.Area(p);
					if (_Mode != MeasureMode.Feature)
						dA = Math.Abs(dA);
				}
				dSumLen += dL;
				dSumArea += dA;
				if (bFirst)
				{
					dLen = dL;
					dArea = dA;
					if ((_Mode != MeasureMode.Feature) && (dLen > 0))
					{
						PointCollection PC = PCList[0];
						int i = PC.Count - 1;
						if (!bLine)
							i = PC.Count - 2;
						MapPoint pt1 = PC[i - 1];
						MapPoint pt2 = PC[i];
						dSegLen = clsGeometryHelper.Distance(pt1, pt2);
					}
					bFirst = false;
				}
				if (!_bShowSum)
					break;

			}

			// Build text

			string sTxt = "";
			switch (_Mode)
			{
				case MeasureMode.Line:
					sTxt = "Line measurement (Planar)";
					break;
				case MeasureMode.Poly:
					sTxt = "Area measurement";
					break;
				case MeasureMode.Feature:
					if (bLine)
						sTxt = "Line feature (Planar)";
					else
						sTxt = "Polygon feature";
					break;
			}
			if (_Mode != MeasureMode.Feature)
			{
				sTxt += "\n Segment: ";
				sTxt += _gh.ConvertLinearUnit(dSegLen, _MapUnits, _DisplayUnits).ToString("N6");
				sTxt += " " + this.DisplayUnitToString();
			}
			if (bLine)
			{
				sTxt += "\nLength: ";
				sTxt += _gh.ConvertLinearUnit(dLen, _MapUnits, _DisplayUnits).ToString("N6");
				sTxt += " " + this.DisplayUnitToString();
				if (_bShowSum)
				{
					sTxt += "\n\nSum Lengths: ";
					sTxt += _gh.ConvertLinearUnit(dSumLen, _MapUnits, _DisplayUnits).ToString("N6");
					sTxt += " " + this.DisplayUnitToString();
				}
			}
			else
			{
				sTxt += "\nPerimeter: ";
				sTxt += _gh.ConvertLinearUnit(dLen, _MapUnits, _DisplayUnits).ToString("N6");
				sTxt += " " + this.DisplayUnitToString() + "\nArea: ";
				sTxt += _gh.ConvertAreaUnit(dArea, _MapAreaUnits, _DisplayAreaUnits).ToString("N6");
				sTxt += " " + this.AreaUnitToString();
				if (_bShowSum)
				{
					sTxt += "\n\nSum Perimeters: ";
					sTxt += _gh.ConvertLinearUnit(dSumLen, _MapUnits, _DisplayUnits).ToString("N6");
					sTxt += " " + this.DisplayUnitToString() + "\nSum Areas: ";
					sTxt += _gh.ConvertAreaUnit(dSumArea, _MapAreaUnits, _DisplayAreaUnits).ToString("N6");
					sTxt += " " + this.AreaUnitToString();
				}
			}
			txtResults.Text = sTxt;
			
		}

		private void UpdatePoint(MapPoint pt)
		{
			string sTxt = "Point feature\n";
			sTxt += "X: " + _gh.ConvertLinearUnit(pt.X, _MapUnits, _DisplayUnits);
			sTxt += " " + this.DisplayUnitToString() + "\n";
			sTxt += "Y: " + _gh.ConvertLinearUnit(pt.Y, _MapUnits, _DisplayUnits);
			sTxt += " " + this.DisplayUnitToString();
			txtResults.Text = sTxt;
		}

		private string DisplayUnitToString()
		{
			string sUnits = "";
			switch (_DisplayUnits)
			{
				case esriUnits.esriCentimeters:
					sUnits = "Centimeters";
					break;
				case esriUnits.esriDecimeters:
					sUnits = "Decimeters";
					break;
				case esriUnits.esriFeet:
					sUnits = "Feet";
					break;
				case esriUnits.esriInches:
					sUnits = "Inches";
					break;
				case esriUnits.esriKilometers:
					sUnits = "Kilometers";
					break;
				case esriUnits.esriMeters:
					sUnits = "Meters";
					break;
				case esriUnits.esriMiles:
					sUnits = "Miles";
					break;
				case esriUnits.esriMillimeters:
					sUnits = "Millimeters";
					break;
				case esriUnits.esriNauticalMiles:
					sUnits = "Nautical Miles";
					break;
				case esriUnits.esriYards:
					sUnits = "Yards";
					break;
			}
			return sUnits;
		}

		private string AreaUnitToString()
		{
			string sUnits = "";
			switch (_DisplayAreaUnits)
			{
				case AreaUnit.Acres:
					sUnits = "Acres";
					break;
				case AreaUnit.Ares:
					sUnits = "Ares";
					break;
				case AreaUnit.Hectares:
					sUnits = "Hectares";
					break;
				case AreaUnit.SquareCentimeters:
					sUnits = "Square Centimeters";
					break;
				case AreaUnit.SquareDecimeters:
					sUnits = "Square Decimeters";
					break;
				case AreaUnit.SquareFeet:
					sUnits = "Square Feet";
					break;
				case AreaUnit.SquareInches:
					sUnits = "Square Inches";
					break;
				case AreaUnit.SquareKilometers:
					sUnits = "Square Kilometers";
					break;
				case AreaUnit.SquareMeters:
					sUnits = "Square Meters";
					break;
				case AreaUnit.SquareMiles:
					sUnits = "Square Miles";
					break;
				case AreaUnit.SquareMillimeters:
					sUnits = "Square Millimeters";
					break;
				case AreaUnit.SquareYards:
					sUnits = "Square Yards";
					break;
			}
			return sUnits;
		}
		#endregion

	}
}
