﻿using System;
using System.Reflection;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;

using FieldApp.Extensibility;

namespace QueryAddin
{

		// Convenient base class for query commands
		#region QueryButton
		public abstract class QueryButton : IMapButton
		{

			private bool _bInitialized = false;
			private bool _bMapEnabled = false;

			protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
			protected string _sButtonContent;
			protected string _sIconPath;
			protected string _sToolTip;

			#region Overridable Methods
			abstract protected void QExecute();
			#endregion

			#region IMapButton Implementation
			public object ButtonContent
			{
				get
				{
					if (_sButtonContent != null)
						return _sButtonContent;
					else if (_sIconPath != null)
						return AddinHelper.GetImage(_MyAssembly, _sIconPath);
					return null;
				}
			}

			public bool IsEnabled
			{
				get
				{
					if (!_bInitialized)
					{
						_bInitialized = true;
						QueryBehavior qb = QueryBehavior.Current;
						if (!qb.IsInitialized)
						{
							qb.Initialized += new EventHandler<EventArgs>(qb_Initialized);
							return false;
						}
						_bMapEnabled = true;
					}
					return _bMapEnabled;
				}
			}

			public string ToolTip
			{
				get { return _sToolTip; }
			}

			public void OnClick()
			{
				this.QExecute();
			}

			public event EventHandler<EventArgs> IsEnabledChanged;
			#endregion

			void qb_Initialized(object sender, EventArgs e)
			{
				_bMapEnabled = true;
				if (IsEnabledChanged != null)
					IsEnabledChanged(this, new EventArgs());
			}

		}
		#endregion

		// Convenient base class for query tools
		#region QueryTool
		public abstract class QueryTool : IMapTool
		{

			private bool _bInitialized = false;
			private bool _bMapEnabled = false;

			protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
			protected string _sButtonContent;
			protected string _sIconPath;
			protected string _sToolTip;
			protected string _sCursorPath;

			#region Overridable Methods
			abstract protected void QInitialized();
			abstract protected void QActivateTool();
			abstract protected void QDeactivateTool();
			#endregion

			#region IMapTool Implementation
			public object ButtonContent
			{
				get
				{
					if (_sButtonContent != null)
						return _sButtonContent;
					else if (_sIconPath != null)
						return AddinHelper.GetImage(_MyAssembly, _sIconPath);
					return null;
				}
			}

			public Cursor MouseCursor
			{
				get
				{
					if (_sCursorPath == null)
						return Cursors.Arrow;
					return AddinHelper.GetCursor(_MyAssembly, _sCursorPath);
				}
			}

			public bool IsEnabled
			{
				get
				{
					if (!_bInitialized)
					{
						_bInitialized = true;
						QueryBehavior qb = QueryBehavior.Current;
						if (!qb.IsInitialized)
						{
							qb.Initialized += new EventHandler<EventArgs>(qb_Initialized);
							return false;
						}
						_bMapEnabled = true;
					}
					return _bMapEnabled;
				}
			}

			public string ToolTip
			{
				get { return _sToolTip; }
			}

			public void Activate()
			{
				this.QActivateTool();
			}

			public void Deactivate()
			{
				this.QDeactivateTool();
			}

			public event EventHandler<EventArgs> IsEnabledChanged;
			#endregion

			void qb_Initialized(object sender, EventArgs e)
			{
				this.QInitialized();
				_bMapEnabled = true;
				if (IsEnabledChanged != null)
					IsEnabledChanged(this, new EventArgs());
			}

		}
		#endregion

		#region QuerySplitTool
		public abstract class QuerySplitTool : IMapTool, IMapSplitButton
		{

			private bool _bInitialized = false;
			private bool _bMapEnabled = false;

			protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
			protected string _sButtonContent;
			protected string _sIconPath;
			protected string _sDropIconPath;
			protected string _sToolTip;
			protected string _sCursorPath;

			#region Overridable Methods
			abstract protected void QInitialized();
			abstract protected void QActivateTool();
			abstract protected void QDeactivateTool();
			abstract protected void QOnItemSelected(IMapMenuItem item);
			#endregion

			#region IMapTool Implementation
			public object ButtonContent
			{
				get
				{
					if (_sButtonContent != null)
						return _sButtonContent;
					else if (_sIconPath != null)
						return AddinHelper.GetImage(_MyAssembly, _sIconPath);
					return null;
				}
			}

			public Cursor MouseCursor
			{
				get
				{
					if (_sCursorPath == null)
						return Cursors.Arrow;
					return AddinHelper.GetCursor(_MyAssembly, _sCursorPath);
				}
			}

			public bool IsEnabled
			{
				get
				{
					if (!_bInitialized)
					{
						_bInitialized = true;
						QueryBehavior qb = QueryBehavior.Current;
						if (!qb.IsInitialized)
						{
							qb.Initialized += new EventHandler<EventArgs>(qb_Initialized);
							return false;
						}
						_bMapEnabled = true;
					}
					return _bMapEnabled;
				}
			}

			public string ToolTip
			{
				get { return _sToolTip; }
			}

			public void Activate()
			{
				this.QActivateTool();
			}

			public void Deactivate()
			{
				this.QDeactivateTool();
			}

			public event EventHandler<EventArgs> IsEnabledChanged;
			#endregion

			#region IMapSplitButton implementation
			public object DropButtonContent
			{
				get 
				{
					if (_sDropIconPath != null)
						return AddinHelper.GetImage(_MyAssembly, _sDropIconPath);
					return null;
				}
			}

			public void OnItemSelected(IMapMenuItem item)
			{
				if (!_bMapEnabled)
					return;
				this.QOnItemSelected(item);
				this.SelectionChanged(this, new EventArgs());
			}

			public event EventHandler<EventArgs> SelectionChanged;
			#endregion

			void qb_Initialized(object sender, EventArgs e)
			{
				this.QInitialized();
				_bMapEnabled = true;
				if (IsEnabledChanged != null)
					IsEnabledChanged(this, new EventArgs());
			}

		}
		#endregion

		#region QueryMenuItem
		public abstract class QueryMenuItem : IMapMenuItem
		{

			private bool _bInitialized = false;
			private bool _bMapEnabled = false;

			protected Assembly _MyAssembly; // _MyAssembly = Assembly.GetExecutingAssembly();
			protected string _sLabel;
			protected string _sIconPath;
			protected bool _bIsCheckable;
			protected bool _bIsChecked;

			#region Overridable Methods
			abstract protected void QExecute();
			#endregion

			#region IMapMenuItem implementation
			public Image Icon
			{
				get
				{
					if (_sIconPath != null)
						return AddinHelper.GetImage(_MyAssembly, _sIconPath);
					return null;
				}
			}

			public bool IsCheckable
			{
				get { return _bIsCheckable; }
			}

			public bool IsChecked
			{
				get { return _bIsChecked; }
			}

			public bool IsEnabled
			{
				get
				{
					if (!_bInitialized)
					{
						_bInitialized = true;
						QueryBehavior qb = QueryBehavior.Current;
						if (!qb.IsInitialized)
						{
							qb.Initialized += new EventHandler<EventArgs>(qb_Initialized);
							return false;
						}
						_bMapEnabled = true;
					}
					return _bMapEnabled;
				}
			}

			public event EventHandler<EventArgs> IsEnabledChanged;

			public string Label
			{
				get { return _sLabel; }
			}

			public void OnClick()
			{
				this.QExecute();
			}
			#endregion

			void qb_Initialized(object sender, EventArgs e)
			{
				_bMapEnabled = true;
				if (IsEnabledChanged != null)
					IsEnabledChanged(this, new EventArgs());
			}

		}
		#endregion

}
