﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows;

namespace FieldApp.Extensibility
{

	public class InstallHelper
	{

		public const string RegBase = @"HKEY_LOCAL_MACHINE\Software\UNS\FieldApp";
		private const string UpdatePathEntryName = "UpdatePath";
		public static string UpdatePath { get; private set; }
		public string Configuration { get; set; }

		public bool GetUpdatePath()
		{

			object oVal;

			// Get update path from registry

			UpdatePath = "";
			oVal = Registry.GetValue(RegBase, UpdatePathEntryName, null);
			if (oVal == null)
			{
				string sMsg = "Update path not found in registry.";
				MessageBoxResult mbResult = MessageBox.Show(sMsg, "FieldApp", MessageBoxButton.OK);
				return false;
			}
			UpdatePath = oVal.ToString();
			return Directory.Exists(UpdatePath);

		}

		public bool GetConfiguration()
		{

			// Check for access to update folder

			string sConfig = "";
			string sMsg = "";
			if (!Directory.Exists(UpdatePath))
			{
				sMsg = "Update folder not found.  Please connect to the network.";
				MessageBox.Show(sMsg, "FieldApp", MessageBoxButton.OK, MessageBoxImage.Exclamation);
				return false;
			}

			// Select configuration file

			string sConfigPath = UpdatePath + "\\Config";
			List<string> ConfigList = new List<string>();
			foreach (string sConfigFileName in Directory.GetFiles(sConfigPath))
				ConfigList.Add(Path.GetFileNameWithoutExtension(sConfigFileName));
			PickListDialog dlg = new PickListDialog();
			sConfig = dlg.Open(ConfigList, "Choose Configuration");
			if (sConfig == "")
				return false;
			Configuration = sConfig;
			sMsg = "Would you like to create a desktop shortcut for this configuration?";
			MessageBoxResult mbResult = MessageBox.Show(sMsg, sConfig, MessageBoxButton.YesNo);
			if (mbResult == MessageBoxResult.Yes)
				this.CreateDesktopShortcut();
			return true;

		}

		public void CheckForApplicationUpdates()
		{

			// Check for updates

			bool bUpdate = false;
			Assembly a = Assembly.GetExecutingAssembly();
			string sLocalPath = Path.GetDirectoryName(a.Location);
			string sRemotePath = UpdatePath + "\\FieldApp";
			foreach (string sRemoteFileName in Directory.GetFiles(sRemotePath))
			{
				string sLocalFileName = sRemoteFileName.Replace(sRemotePath, sLocalPath);
				if (sLocalFileName.ToLower().Contains("fieldappupdater.exe"))
					continue;
				bool bResult = this.CheckForUpdate(sLocalFileName, sRemoteFileName, true);
				if (bResult)
				{
					bUpdate = true;
					break;
				}
			}
			if (bUpdate)
			{

				string sMsg = "Updates have been detected.  Would you like to install them?";
				MessageBoxResult mbResult = MessageBox.Show(sMsg, "FieldApp", MessageBoxButton.YesNo);
				if (mbResult != MessageBoxResult.Yes)
					return;

				// Check if updater is updated

				string sUpdater = "FieldAppUpdater.exe";
				string sRemoteFileName = sRemotePath + "\\" + sUpdater;
				string sLocalFileName = sLocalPath + "\\" + sUpdater;
				this.CheckForUpdate(sLocalFileName, sRemoteFileName, false);

				// Launch updater

				if (Configuration != null)
					System.Diagnostics.Process.Start(sLocalFileName, Configuration);
				else
					System.Diagnostics.Process.Start(sLocalFileName);
				Application.Current.Shutdown();

			}

		}

		public void CheckForUpdate(string sLocalFileName, string sUpdateFileName)
		{
			this.CheckForUpdate(sLocalFileName, sUpdateFileName, false);
		}

		private void CreateDesktopShortcut()
		{

			Assembly a = Assembly.GetExecutingAssembly();
			string sLocalFile = a.Location;
			string sLocalPath = Path.GetDirectoryName(sLocalFile);
			string sLocalExe = sLocalPath + "\\FieldApp.exe";
			string sDesktop = Environment.GetFolderPath(Environment.SpecialFolder.CommonDesktopDirectory);
			string sOutPath = sDesktop + "\\FieldApp (" + Configuration + ").lnk";

			// Create shortcut

			using (ShellLink shortcut = new ShellLink())
			{
				shortcut.Target = sLocalExe;
				shortcut.Arguments = Configuration;
				shortcut.WorkingDirectory = sLocalPath;
				shortcut.Description = "FieldApp";
				shortcut.DisplayMode = ShellLink.LinkDisplayMode.edmNormal;
				shortcut.IconPath = sLocalExe;
				shortcut.IconIndex = 0;
				shortcut.Save(sOutPath);
			}

		}

		private bool CheckForUpdate(string sLocalFileName, string sUpdateFileName, bool bCheckOnly)
		{

			bool bUpdate = false;
			if (!File.Exists(sLocalFileName))
				bUpdate = true;
			else
			{
				FileInfo fiUpdate = new FileInfo(sUpdateFileName);
				FileInfo fiLocal = new FileInfo(sLocalFileName);
				if (fiUpdate.LastWriteTime > fiLocal.LastWriteTime)
					bUpdate = true;
			}
			if (bCheckOnly)
				return bUpdate;
			if (!bUpdate)
				return bUpdate;
			try
			{
				string sDir = Path.GetDirectoryName(sLocalFileName);
				if (!Directory.Exists(sDir))
					Directory.CreateDirectory(sDir);
				File.Copy(sUpdateFileName, sLocalFileName, true);
			}
			catch (Exception ex)
			{
				string sErr = "Error updating " + sLocalFileName + ":\n" + ex.Message;
				MessageBox.Show(sErr, "FieldApp", MessageBoxButton.OK, MessageBoxImage.Error);
				bUpdate = false;
			}
			return bUpdate;

		}

	}
}
