﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Local;
using ESRI.ArcGIS.Client.Toolkit;
using ESRI.ArcGIS.Client.Toolkit.Primitives;

namespace FieldApp.Extensibility
{

	// Make this public so that extensions can use it to load the map

	public class MapHelper
	{

		public const string BackgroundEntryName = "Background";
		public const string BackgroundEntryBlackValue = "BLACK";

		private enum mhLayerType
		{
			Invalid,
			TilePackage,
			MapPackage,
			GraphicLayer
		}

		private struct mhLayer
		{
			public Layer Layer;
			public mhLayerType LayerType;
			public string Name;
			public string Path;
			public bool Legend;
			public int CollapseLevel;
			public bool FeatureService;
			public bool Initialized;
			public bool Collapsed;
		}

		private Map _Map;
		private Legend _Legend;
		private IStatus _pStatus;
		private Envelope _MapExtent;
		private double _dMinResolution;
		private List<string> _LayerNames;
		private Dictionary<string, mhLayer> _MapLayers;
		private bool _bLegendInitialized;

		public void Initialize(Map MyMap, Legend MyLegend, IStatus pStatus)
		{
			_Map = MyMap;
			_Legend = MyLegend;
			_pStatus = pStatus;
		}

		public void LoadMap(XElement xMap)
		{

			// Check for background configuration

			bool bBlack = false;
			bool bBlackPackageFound = false;
			object oVal = Registry.GetValue(InstallHelper.RegBase, BackgroundEntryName, null);
			if (oVal != null)
				bBlack = (oVal.ToString().ToUpper() == BackgroundEntryBlackValue);

			// Get map extent if specified

			_MapExtent = null;
			XAttribute xAttr = xMap.Attribute("Extent");
			if (xAttr != null)
			{
				try
				{
					string sExtent = xAttr.Value;
					string[] Tok = sExtent.Split(new char[] { ',', ' ' }, StringSplitOptions.RemoveEmptyEntries);
					Envelope env = new Envelope();
					env.XMin = Convert.ToDouble(Tok[0]);
					env.YMin = Convert.ToDouble(Tok[1]);
					env.XMax = Convert.ToDouble(Tok[2]);
					env.YMax = Convert.ToDouble(Tok[3]);
					_MapExtent = env;
				}
				catch
				{
					_MapExtent = null;
				}
			}

			// Get minimum resolution if specified

			_dMinResolution = 0;
			xAttr = xMap.Attribute("MinimumResolution");
			if (xAttr != null)
			{
				try
				{
					_dMinResolution = Convert.ToDouble(xAttr.Value);
				}
				catch
				{
					_dMinResolution = 0;
				}
			}

			// Collect layer info

			_LayerNames = new List<string>();
			_MapLayers = new Dictionary<string, mhLayer>();
			foreach (XElement xLayer in xMap.Elements())
			{

				// Validate layer

				mhLayerType lt = mhLayerType.Invalid;
				string sLayerType = xLayer.Name.LocalName;
				switch (sLayerType)
				{
					case "TilePackage":
						lt = mhLayerType.TilePackage;
						break;
					case "MapPackage":
						lt = mhLayerType.MapPackage;
						break;
					case "GraphicLayer":
						lt = mhLayerType.GraphicLayer;
						break;
				}
				if (lt == mhLayerType.Invalid)
					continue;
				xAttr = xLayer.Attribute("ID");
				if (xAttr == null)
					continue;
				string sLayerName = xAttr.Value;
				xAttr = xLayer.Attribute("Path");
				string sPath = null;
				if (xAttr != null)
					sPath = xAttr.Value;
				if (lt != mhLayerType.GraphicLayer)
				{
					if (sPath == null)
						continue;
					if (bBlack)
					{

						// Check for black background package

						string sBlackPath = sPath.ToLower().Replace(".mpk", " black.mpk");
						if (File.Exists(sBlackPath))
						{
							sPath = sBlackPath;
							bBlackPackageFound = true;
						}
					}
					if (!File.Exists(sPath))
						continue;
				}

				// Build layer entry

				_LayerNames.Add(sLayerName);
				mhLayer l = new mhLayer();
				l.LayerType = lt;
				l.Name = sLayerName;
				l.Path = sPath;
				l.Legend = true;
				l.CollapseLevel = 0;
				l.FeatureService = false;
				l.Initialized = false;
				l.Collapsed = false;
				xAttr = xLayer.Attribute("Legend");
				if (xAttr != null)
				{
					string sVal = xAttr.Value;
					if (sVal == "False")
						l.Legend = false;
				}
				xAttr = xLayer.Attribute("CollapseLevel");
				if ((l.LayerType == mhLayerType.MapPackage) && (xAttr != null))
				{
					int i = 0;
					try
					{
						i = Convert.ToInt32(xAttr.Value);
					}
					catch
					{
						i = 0;
					}
					l.CollapseLevel = i;
				}
				xAttr = xLayer.Attribute("FeatureService");
				if ((l.LayerType == mhLayerType.MapPackage) && (xAttr != null))
				{
					string sVal = xAttr.Value;
					if (sVal == "True")
						l.FeatureService = true;
				}
				_MapLayers[sLayerName] = l;

			}
			if (_LayerNames.Count == 0)
				return;

			// Set background

			if (bBlackPackageFound)
				_Map.Background = System.Windows.Media.Brushes.Black;

			// Create layers

			_pStatus.Status("Initializing Map...", true);
			foreach (string sLayerName in _LayerNames)
			{
				mhLayer l = _MapLayers[sLayerName];
				switch (l.LayerType)
				{
					case mhLayerType.TilePackage:
						ArcGISLocalTiledLayer ltLyr = new ArcGISLocalTiledLayer(l.Path);
						ltLyr.DisplayName = l.Name;
						ltLyr.ID = l.Name;
						ltLyr.Visible = true;
						ltLyr.Initialized += new EventHandler<EventArgs>(Lyr_Initialized);
						l.Layer = ltLyr;
						_MapLayers[sLayerName] = l;
						break;
					case mhLayerType.GraphicLayer:
						GraphicsLayer grLyr = new GraphicsLayer();
						grLyr.DisplayName = l.Name;
						grLyr.ID = l.Name;
						grLyr.Visible = true;
						l.Layer = grLyr;
						_MapLayers[sLayerName] = l;
						break;
					default:
						if (l.FeatureService)
						{
							LocalFeatureService.GetServiceAsync(l.Path,
								Svc =>
								{
									if (Svc.Error != null)
									{
										string sMsg = "Error creating local feature service " + l.Name + ":\n";
										sMsg += Svc.Error.Message;
										System.Windows.MessageBox.Show(sMsg);
										return;
									}
									this.CreateLocalMapLayer(Svc, l.Name);
								});
						}
						else
						{
							LocalMapService.GetServiceAsync(l.Path,
								Svc =>
								{
									if (Svc.Error != null)
									{
										string sMsg = "Error creating local map service " + l.Name + ":\n";
										sMsg += Svc.Error.Message;
										System.Windows.MessageBox.Show(sMsg);
										return;
									}
									this.CreateLocalMapLayer(Svc, l.Name);
								});
						}
						break;
				}
			}
			if (this.LayersCreated())
				this.AddLayers();

		}

		void CreateLocalMapLayer(LocalMapService Svc, string sLayerName)
		{
			ArcGISLocalDynamicMapServiceLayer Lyr = new ArcGISLocalDynamicMapServiceLayer(Svc);
			mhLayer l = _MapLayers[sLayerName];
			Lyr.DisplayName = l.Name;
			Lyr.ID = l.Name;
			Lyr.Visible = true;
			Lyr.Initialized += new EventHandler<EventArgs>(Lyr_Initialized);
			l.Layer = Lyr;
			_MapLayers[l.Name] = l;
			if (this.LayersCreated())
				this.AddLayers();
		}

		void Lyr_Initialized(object sender, EventArgs e)
		{
			Layer Lyr = (Layer)sender;
			string sLayerID = Lyr.ID;
			mhLayer l = _MapLayers[sLayerID];
			l.Initialized = true;
			_MapLayers[sLayerID] = l;
			if (this.LayersInitialized())
				this.Finished();
		}

		private bool LayersCreated()
		{
			foreach (string sLayerName in _LayerNames)
			{
				mhLayer l = _MapLayers[sLayerName];
				if (l.Layer == null)
					return false;
			}
			return true;
		}

		private bool LayersInitialized()
		{
			foreach (string sLayerName in _LayerNames)
			{
				mhLayer l = _MapLayers[sLayerName];
				if (l.LayerType == mhLayerType.GraphicLayer)
					continue;
				if (!l.Initialized)
					return false;
			}
			return true;
		}

		private void AddLayers()
		{
			if (_MapExtent != null)
				_Map.Extent = _MapExtent;
			List<string> LegendIDs = new List<string>();
			foreach (string sLayerName in _LayerNames)
			{
				mhLayer l = _MapLayers[sLayerName];
				_Map.Layers.Add(l.Layer);
				if (l.Legend)
					LegendIDs.Add(sLayerName);
			}
			if (_Legend != null)
			{
				if (LegendIDs.Count > 1)
					LegendIDs.Reverse();
				_bLegendInitialized = false;
				_Legend.Map = _Map;
				_Legend.LayerIDs = LegendIDs.ToArray();
				_Legend.Refreshed += new EventHandler<Legend.RefreshedEventArgs>(_Legend_Refreshed);
			}
		}

		void _Legend_Refreshed(object sender, Legend.RefreshedEventArgs e)
		{
			if (_bLegendInitialized)
				return;
			foreach (LegendItemViewModel lm in _Legend.LayerItemsSource)
			{
				string sLayerName = lm.Label;
				mhLayer l = _MapLayers[sLayerName];
				int iCollapseLevel = l.CollapseLevel;
				if (iCollapseLevel < 0)
					continue;
				if (l.Collapsed)
					continue;
				if (this.CollapseItems(lm, iCollapseLevel, 0))
				{
					// Entry is populated
					l.Collapsed = true;
					_MapLayers[sLayerName] = l;
				}
			}
			bool bInit = true;
			foreach (string sLayerName in _LayerNames)
			{
				mhLayer l = _MapLayers[sLayerName];
				int iCollapseLevel = l.CollapseLevel;
				if (iCollapseLevel < 0)
					continue;
				if (l.Collapsed)
					continue;
				bInit = false;
			}
			if (bInit)
				_bLegendInitialized = true;
			
		}

		private bool CollapseItems(LegendItemViewModel lm, int iCollapseLevel, int iCurrentLevel)
		{
			if (lm.LayerItemsSource == null)
				return false;
			if ((iCurrentLevel == 0) && (lm.LayerItemsSource.Count() == 0))
				return false;
			if (iCurrentLevel >= iCollapseLevel)
				lm.IsExpanded = false;
			foreach (LegendItemViewModel lmChild in lm.LayerItemsSource)
				this.CollapseItems(lmChild, iCollapseLevel, iCurrentLevel + 1);
			return true;
		}

		private void Finished()
		{
			if (_dMinResolution > 0)
				_Map.MinimumResolution = _dMinResolution;
			_pStatus.Status("OK", false);
		}
		

	}
}
