﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Xml;
using System.Xml.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Toolkit;

namespace FieldApp.Extensibility
{

	public class MapApplication : IStatus
	{


		public string Configuration { get; private set; }
		public UserControl Layout { get; private set; }
		public ResourceDictionary Resources { get; private set; }
		public Map Map { get; private set; }
		public Legend Legend { get; private set; }
		public ItemsControl MainMenu { get; private set; }
		public ItemsControl MainToolBar { get; private set; }

		private static MapApplication _current;
		private string _sExePath;
		private bool _bUpdate;
		private string _sUpdatePath;
		private IMainWindow _pMainWindow;
		private InstallHelper _ih;
		private Dictionary<string, string> _NameSpaces;
		private TextBlock _tbStatus;
		private ProgressBar _pbBusy;
		private Dictionary<string, Assembly> _Assemblies;
		private List<IBehavior> _Behaviors;
		private List<MapTool> _Tools;
		private MapHelper _mh;
		private MapTool _CurrentTool;

		internal MapApplication() { }

		public static MapApplication Current
		{
			get
			{
				if (_current == null)
				{
					_current = new MapApplication();
					_current.Initialize();
				}
				return _current;
			}
		}

		public event EventHandler<EventArgs> ToolActivated;

		public void DeactivateCurrentTool()
		{
			if (_CurrentTool == null)
				return;
			_CurrentTool.Deactivate();
		}

		public void AddWindow(ParentWindow parentWindow)
		{
			_pMainWindow.AddWindow(parentWindow);
		}

		public void ToggleBackground()
		{

			// Get and toggle background setting

			bool bBlack = false;
			object oVal = Registry.GetValue(InstallHelper.RegBase, MapHelper.BackgroundEntryName, null);
			if (oVal != null)
				bBlack = (oVal.ToString().ToUpper() == MapHelper.BackgroundEntryBlackValue);
			bBlack = !bBlack;

			// Save background setting

			string sConfig;
			if (bBlack)
				sConfig = MapHelper.BackgroundEntryBlackValue;
			else
				sConfig = "NOT " + MapHelper.BackgroundEntryBlackValue;
			Registry.SetValue(InstallHelper.RegBase, MapHelper.BackgroundEntryName, sConfig);
			string sMsg = "Restart application for changes to take effect.";
			MessageBox.Show(sMsg, "Toggle Map Display", MessageBoxButton.OK, MessageBoxImage.Information);

		}

		public string GetOutputFile(string sExt)
		{
			Microsoft.Win32.SaveFileDialog sfDialog = new Microsoft.Win32.SaveFileDialog();
			sfDialog.Filter = sExt + " files (*." + sExt + ")|*." + sExt + "|All files (*.*)|*.*";
			if (sfDialog.ShowDialog() != true)
				return "";
			return sfDialog.FileName;
		}

		public void ExportMap(Map MyMap, string sOutFile)
		{
			RenderTargetBitmap rtBitmap = this.GetMapImage(MyMap);
			BmpBitmapEncoder bmpEncoder = new BmpBitmapEncoder();
			bmpEncoder.Frames.Add(BitmapFrame.Create(rtBitmap));
			using (Stream strOut = new FileStream(sOutFile, FileMode.Create))
			{
				bmpEncoder.Save(strOut);
			}
			this.Message("File saved.");
		}

		public void CopyMap(Map MyMap)
		{
			RenderTargetBitmap rtBitmap = this.GetMapImage(MyMap);
			Clipboard.SetImage(rtBitmap);
			this.Message("Map copied.");
		}

		public void Exit()
		{
			_pMainWindow.Exit();
		}

		#region IStatus implementation
		public void Status(string sMsg)
		{
			if (_tbStatus == null)
				return;
			_tbStatus.Text = sMsg;
			_tbStatus.InvalidateVisual();
			// This is a WPF equivalent of Application.DoEvents()
			Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, new ThreadStart(delegate { }));
		}

		public void Status(string sMsg, bool bBusy)
		{
			if (_pbBusy == null)
				return;
			_pbBusy.IsIndeterminate = true;
			if (bBusy)
			{
				_pbBusy.Visibility = System.Windows.Visibility.Visible;
			}
			else
			{
				_pbBusy.Visibility = System.Windows.Visibility.Hidden;
			}
			this.Status(sMsg);
		}

		public void Message(string sMsg)
		{
			MessageBox.Show(sMsg, "Field App", MessageBoxButton.OK, MessageBoxImage.Information);
		}

		public void Error(string sErr)
		{
			if (_tbStatus != null)
				_tbStatus.Text = "ERROR";
			if (_pbBusy != null)
				_pbBusy.Visibility = System.Windows.Visibility.Hidden;
			MessageBox.Show(sErr, "FieldApp", MessageBoxButton.OK, MessageBoxImage.Error);
		}
		#endregion

		#region Initialization
		private void Initialize()
		{
			try
			{

				// Get main window

				Application app = Application.Current;
				Window mw = app.MainWindow;
				if (!(mw is IMainWindow))
					return; // This should never happen
				_pMainWindow = (IMainWindow)mw;

				// Set exe path

				Assembly a = Assembly.GetExecutingAssembly();
				_sExePath = Path.GetDirectoryName(a.Location);

				// Check for update path

				_ih = new InstallHelper();
				_bUpdate = _ih.GetUpdatePath();
				if (_bUpdate)
					_sUpdatePath = InstallHelper.UpdatePath;

				// Check configuration
				// (Must be connected to network if configuration is not provided at startup)

				Configuration = _pMainWindow.ConfigurationFile;
				if (Configuration != "")
					_ih.Configuration = Configuration;
				else
				{
					if (!_ih.GetConfiguration())
						return;
					Configuration = _ih.Configuration;
				}

				// Check for updates

				if (_bUpdate)
				{

					// Application updates

					_ih.CheckForApplicationUpdates();

					// Configuration updates

					string sLocalConfig = _sExePath + "\\Config\\" + Configuration + ".xml";
					string sRemoteConfig = _sUpdatePath + "\\Config\\" + Configuration + ".xml";
					_ih.CheckForUpdate(sLocalConfig, sRemoteConfig);

				}

				// Load configuration

				this.LoadConfiguration();

			}
			catch (Exception ex)
			{
				string sErr = "Error initializing MapApplication:\n" + ex.Message + "\n" + ex.StackTrace;
				this.Error(sErr);
			}
		}

		private void LoadConfiguration()
		{

			// Load Config File

			string sConfigFilePath = _sExePath + "\\Config\\" + Configuration + ".xml";
			XDocument xDoc = XDocument.Load(sConfigFilePath);
			XElement xRoot = xDoc.Root;

			// Get Namespace Declarations
			// e.g. xmlns:TestAddin="clr-namespace:TestAddin;assembly=TestAddin"

			_NameSpaces = new Dictionary<string, string>();
			foreach (XAttribute xAttr in xRoot.Attributes())
			{
				string sName = xAttr.Name.LocalName;
				string sVal = xAttr.Value;
				_NameSpaces[sName] = sVal;
			}

			// Process Elements

			foreach (XElement xElement in xRoot.Elements())
			{

				string sName = xElement.Name.LocalName;
				switch (sName)
				{
					case "TitleText":
						_pMainWindow.SetTitle(xElement.Value);
						break;
					case "Layout":
						this.LoadLayout(xElement.Value);
						break;
					case "Map":
						this.LoadMap(xElement);
						break;
					case "Extensions":
						this.LoadExtensions(xElement);
						break;
					case "Behaviors":
						this.LoadBehaviors(xElement);
						break;
					case "MenuItems":
						this.LoadMenuItems(xElement);
						break;
					case "ToolBarItems":
						this.LoadToolBarItems(xElement);
						break;
				}
			}

		}

		private void LoadLayout(string sLayoutName)
		{
			string sLayoutPath = _sExePath + "\\Layouts\\" + sLayoutName + ".xaml";
			if (_bUpdate)
			{
				string s_sUpdatePath = _sUpdatePath + "\\Layouts\\" + sLayoutName + ".xaml";
				_ih.CheckForUpdate(sLayoutPath, s_sUpdatePath);
			}
			FileStream xamlFile = new FileStream(sLayoutPath, FileMode.Open, FileAccess.Read);
			Layout = XamlReader.Load(xamlFile) as UserControl;
			Resources = Layout.Resources;
			this.GetChildren(Layout);
			if (Map == null)
				throw new Exception("Could not find map in layout");
			_pMainWindow.SetLayout(Layout);
		}

		private void LoadMap(XElement xMap)
		{
			_mh = new MapHelper();
			_mh.Initialize(Map, Legend, this);
			_mh.LoadMap(xMap);
		}

		private void LoadExtensions(XElement xExtensions)
		{

			_Assemblies = new Dictionary<string, Assembly>();
			bool bDependency = false;
			foreach (XElement xExtension in xExtensions.Elements())
			{
				string sPathName = xExtension.Attribute("Path").Value;
				string sExtensionPath = _sExePath + "\\Extensions\\" + sPathName + ".dll";
				if (_bUpdate)
				{
					string s_sUpdatePath = _sUpdatePath + "\\Extensions\\" + sPathName + ".dll";
					_ih.CheckForUpdate(sExtensionPath, s_sUpdatePath);
				}
				XElement xDependencies = xExtension.Element("Extension.Dependencies");
				if (xDependencies != null)
				{
					foreach (XElement xDependency in xDependencies.Elements())
					{
						string sDependencyName = xDependency.Attribute("Path").Value;
						string sDependencyPath = _sExePath + "\\Extensions\\Dependencies\\" + sDependencyName + ".dll";
						if (_bUpdate)
						{
							string s_sUpdatePath = _sUpdatePath + "\\Extensions\\Dependencies\\" + sDependencyName + ".dll";
							_ih.CheckForUpdate(sDependencyPath, s_sUpdatePath);
						}
						bDependency = true;
						Assembly d = Assembly.LoadFrom(sDependencyPath);
						_Assemblies[sDependencyName] = d; // Keep it alive?
					}
				}
				Assembly a = Assembly.LoadFrom(sExtensionPath);
				_Assemblies[sPathName] = a;
			}
			if (!bDependency)
				return;


		}

		private void LoadBehaviors(XElement xBehaviors)
		{

			_Behaviors = new List<IBehavior>();
			foreach (XElement xBehavior in xBehaviors.Elements())
			{
				string sClassName = xBehavior.Attribute("Class").Value;
				object obj = this.CreateInstance(sClassName);
				if (obj == null)
					continue;
				if (!(obj is IBehavior))
					continue;
				IBehavior pBehavior = (IBehavior)obj;
				XElement xConfigData = xBehavior.Element("Behavior.ConfigData");
				if (xConfigData != null)
				{
					string sConfigData = xConfigData.Value;
					if (string.IsNullOrEmpty(sConfigData))
						sConfigData = xConfigData.ToString(); // Contains XML
					if (pBehavior is IConfigurable)
					{
						IConfigurable pConfig = (IConfigurable)pBehavior;
						pConfig.LoadConfiguration(sConfigData);
					}
				}
				pBehavior.OnAttached();
				_Behaviors.Add(pBehavior);
			}

		}

		private void LoadMenuItems(XElement xMenuItems)
		{

			if (MainMenu == null)
				return;
			this.AddChildMenuItems(MainMenu, xMenuItems);

		}

		private void LoadToolBarItems(XElement xToolBarItems)
		{

			if (MainToolBar == null)
				return;
			_Tools = new List<MapTool>();
			foreach (XElement xToolBarItem in xToolBarItems.Elements())
			{
				string sItemName = xToolBarItem.Name.LocalName;
				object obj = null;
				if (xToolBarItem.Attribute("Class") != null)
				{
					string sClassName = xToolBarItem.Attribute("Class").Value;
					obj = this.CreateInstance(sClassName);
					if (obj == null)
						continue;
				}
				switch (sItemName)
				{
					case "Separator":
						Separator sep = new Separator();
						MainToolBar.Items.Add(sep);
						continue;
					case "Button":
						if (obj == null)
							continue;
						if (!(obj is IMapButton))
							continue;
						IMapButton pButton = (IMapButton)obj;
						MapButton btn = new MapButton(pButton);
						MainToolBar.Items.Add(btn);
						continue;
					case "Tool":
						if (obj == null)
							continue;
						if (!(obj is IMapTool))
							continue;
						IMapTool pTool = (IMapTool)obj;
						MapTool tool = new MapTool(pTool);
						tool.Activated += new EventHandler<EventArgs>(tool_Activated);
						MainToolBar.Items.Add(tool);
						_Tools.Add(tool);
						continue;
					case "SplitButton":
						if (obj == null)
							continue;
						if (!(obj is IMapSplitButton))
							continue;
						Grid gr = new Grid();
						ColumnDefinition coldef = new ColumnDefinition();
						gr.ColumnDefinitions.Add(coldef);
						if (obj is IMapButton)
						{
							IMapButton pSplitButton = (IMapButton)obj;
							MapButton spbtn = new MapButton(pSplitButton);
							if (!double.IsNaN(spbtn.Width))
								coldef.Width = new GridLength(spbtn.Width);
							gr.Children.Add(spbtn);
							Grid.SetColumn(spbtn, 0);
						}
						else if (obj is IMapTool)
						{
							IMapTool pSplitTool = (IMapTool)obj;
							MapTool sptool = new MapTool(pSplitTool);
							sptool.Activated += new EventHandler<EventArgs>(tool_Activated);
							if (!double.IsNaN(sptool.Width))
								coldef.Width = new GridLength(sptool.Width);
							gr.Children.Add(sptool);
							Grid.SetColumn(sptool, 0);
							_Tools.Add(sptool);
						}
						else
							continue;
						IMapSplitButton pParent = (IMapSplitButton)obj;
						XElement xMenuItems = xToolBarItem.Element("SplitButton.Items");
						if (xMenuItems == null)
							continue;
						ContextMenu contextMenu = new ContextMenu();
						this.AddChildMenuItems(contextMenu, xMenuItems);
						MapDropButton drbtn = new MapDropButton(pParent, contextMenu);
						coldef = new ColumnDefinition();
						if (!double.IsNaN(drbtn.Width))
							coldef.Width = new GridLength(drbtn.Width);
						gr.ColumnDefinitions.Add(coldef);
						gr.Children.Add(drbtn);
						Grid.SetColumn(drbtn, 1);
						MainToolBar.Items.Add(gr);
						continue;
					case "ComboBox":
						if (obj == null)
							continue;
						if (!(obj is IMapComboBox))
							continue;
						IMapComboBox pComboBox = (IMapComboBox)obj;
						MapComboBox cb = new MapComboBox(pComboBox);
						MainToolBar.Items.Add(cb);
						continue;
				}
				
			}

		}

		private void GetChildren(FrameworkElement element)
		{
			System.Collections.IEnumerable pEnum = LogicalTreeHelper.GetChildren(element);
			foreach (object o in pEnum)
			{
				if (o is Map)
					Map = (Map)o;
				else if (o is Legend)
					Legend = (Legend)o;
				else if (o is ItemsControl)
				{
					ItemsControl ic = (ItemsControl)o;
					if (ic.Name == "MyMenu")
					{
						MainMenu = ic;
						continue;
					}
					else if (ic.Name == "MyToolBar")
					{
						MainToolBar = ic;
						continue;
					}
					else
						this.GetChildren((FrameworkElement)o);
				}
				else if (o is TextBlock)
				{
					TextBlock tb = (TextBlock)o;
					if (tb.Name == "MyStatusTextBlock")
						_tbStatus = tb;
				}
				else if (o is ProgressBar)
				{
					ProgressBar pb = (ProgressBar)o;
					if (pb.Name == "MyBusyProgressBar")
						_pbBusy = pb;
				}
				else if (o is FrameworkElement)
					this.GetChildren((FrameworkElement)o);
			}
		}

		private object CreateInstance(string sClassID)
		{

			// e.g. "TestAddin:TestBehavior"

			object obj = null;
			try
			{
				string[] Tok = sClassID.Split(':');
				string sNameSpaceKey = Tok[0];
				string sClass = Tok[1];
				string sNameSpaceVal = _NameSpaces[sNameSpaceKey]; // e.g. "clr-namespace:TestAddin;assembly=TestAddin"
				Tok = sNameSpaceVal.Split(';');
				string sNameSpace = Tok[0];
				string sAssembly = Tok[1];
				Tok = sNameSpace.Split(':');
				sNameSpace = Tok[1];
				Tok = sAssembly.Split('=');
				sAssembly = Tok[1];
				Assembly a = _Assemblies[sAssembly];
				string sClassName = sNameSpace + "." + sClass;
				obj = a.CreateInstance(sClassName);
			}
			catch
			{
				this.Error("Invalid class id: '" + sClassID + "'");
			}
			return obj;

		}

		private void AddChildMenuItems(ItemsControl ic, XElement xMenuItems)
		{
			foreach (XElement xMenuItem in xMenuItems.Elements())
			{
				string sItemName = xMenuItem.Name.LocalName;
				if (sItemName == "Separator")
				{
					Separator sep = new Separator();
					ic.Items.Add(sep);
					continue;
				}
				else if (sItemName != "MenuItem")
					continue;
				string sClassName = xMenuItem.Attribute("Class").Value;
				object obj = this.CreateInstance(sClassName);
				if (obj == null)
					continue;
				if (!(obj is IMapMenuItem))
					continue;
				IMapMenuItem pMenuItem = (IMapMenuItem)obj;
				MapMenuItem miChild = new MapMenuItem(pMenuItem);
				ic.Items.Add(miChild);
				XElement xChild = xMenuItem.Element("MenuItem.Items");
				if (xChild != null)
					this.AddChildMenuItems(miChild, xChild);
			}
		}
		#endregion

		#region Event Handlers
		private void tool_Activated(object sender, EventArgs e)
		{
			MapTool newTool = (MapTool)sender;
			foreach (MapTool tool in _Tools)
			{
				if (tool == newTool)
					continue;
				tool.Deactivate();
			}
			Map.Cursor = newTool.MouseCursor;
			_CurrentTool = newTool;
			if (this.ToolActivated != null)
				this.ToolActivated(this, new EventArgs());
		}
		#endregion

		private RenderTargetBitmap GetMapImage(Map MyMap)
		{

			// Save map transform

			System.Windows.Media.Transform t = MyMap.LayoutTransform;
			Rect r = System.Windows.Controls.Primitives.LayoutInformation.GetLayoutSlot(MyMap);
			MyMap.LayoutTransform = null;
			Size sz = new Size(MyMap.ActualWidth, MyMap.ActualHeight);
			MyMap.Measure(sz);
			MyMap.Arrange(new Rect(sz));

			// Output map

			RenderTargetBitmap rtBitmap = new RenderTargetBitmap(
				(int)sz.Width, (int)sz.Height, 96d, 96d,
				System.Windows.Media.PixelFormats.Pbgra32);
			rtBitmap.Render(MyMap);

			// Restore map transform

			MyMap.Arrange(r);
			MyMap.LayoutTransform = t;

			return rtBitmap;

		}

	}
}
